from __future__ import annotations

from typing import Optional

from typing_extensions import TypedDict

from .. import nn
from .._typing import DTypeLike, DynamicShapeLike, Tensor

class ModelSummary(TypedDict):
    total_params: int
    trainable_params: int

def summary(
    net: nn.Layer,
    input_size: Optional[list[DynamicShapeLike]] = ...,
    dtypes: Optional[DTypeLike] = ...,
    input: Optional[Tensor] = ...,
) -> ModelSummary: ...



from __future__ import annotations

from typing import Optional

from ...fluid.initializer import UniformInitializer

class Uniform(UniformInitializer):
    def __init__(self, low: float = ..., high: float = ..., name: Optional[str] = ...) -> None: ...



from __future__ import annotations

from typing import Any, Optional

import numpy as np
import numpy.typing as npt

from ...fluid.initializer import NumpyArrayInitializer

class Assign(NumpyArrayInitializer):
    def __init__(
        self,
        value: npt.NDArray[Any],
        name: Optional[str] = ...,
    ) -> None: ...



from __future__ import annotations

from collections.abc import Sequence
from typing import Any, Optional, TypeVar

import numpy as np
import numpy.typing as npt
from PIL.Image import Image as PILImage
from typing_extensions import Literal

from ..._typing import DataLayoutImage, NumbericSequence, Tensor
from ..._typing.basic import IntSequence

_DataT = TypeVar("_DataT", bound=Tensor | PILImage | npt.NDArray[Any])
_InterpolationPil = Literal["nearest", "bilinear", "bicubic", "lanczos", "hamming"]
_InterpolationCv2 = Literal["nearest", "bilinear", "area", "bicubic", "lanczos"]

class Compose:
    transforms: Any = ...
    def __init__(
        self,
        transforms: list[BaseTransform] | tuple[BaseTransform, ...],
    ) -> None: ...
    def __call__(self, data: _DataT) -> _DataT: ...

class BaseTransform:
    keys: Any = ...
    params: Any = ...
    def __init__(self, keys: Optional[Sequence[str]] = ...) -> None: ...
    def __call__(self, inputs: _DataT) -> _DataT: ...

class ToTensor(BaseTransform):
    data_format: Any = ...
    def __init__(self, data_format: str = ..., keys: Optional[Any] = ...) -> None: ...
    def __call__(self, inputs: _DataT) -> Tensor: ...  # pyright: ignore [reportInvalidTypeVarUse]

class Resize(BaseTransform):
    size: Any = ...
    interpolation: Any = ...
    def __init__(
        self,
        size: int | list[int] | tuple[int, ...],
        interpolation: _InterpolationPil | _InterpolationCv2 = ...,
        keys: Optional[Sequence[str]] = ...,
    ) -> None: ...
    def __call__(self, inputs: _DataT) -> _DataT: ...

class RandomResizedCrop(BaseTransform):
    size: Any = ...
    scale: Any = ...
    ratio: Any = ...
    interpolation: Any = ...
    def __init__(
        self,
        size: int | list[int] | tuple[int, ...],
        scale: list[float] | tuple[float, ...] = ...,
        ratio: list[float] | tuple[float, ...] = ...,
        interpolation: _InterpolationPil | _InterpolationCv2 = ...,
        keys: Optional[Sequence[str]] = ...,
    ) -> None: ...

class CenterCrop(BaseTransform):
    size: Any = ...
    def __init__(
        self,
        size: int | list[int] | tuple[int, ...],
        keys: Optional[Sequence[str]] = ...,
    ) -> None: ...

class RandomHorizontalFlip(BaseTransform):
    prob: Any = ...
    def __init__(
        self,
        prob: float = ...,
        keys: Optional[Sequence[str]] = ...,
    ) -> None: ...

class RandomVerticalFlip(BaseTransform):
    prob: Any = ...
    def __init__(
        self,
        prob: float = ...,
        keys: Optional[Sequence[str]] = ...,
    ) -> None: ...

class Normalize(BaseTransform):
    mean: Any = ...
    std: Any = ...
    data_format: Any = ...
    to_rgb: Any = ...
    def __init__(
        self,
        mean: NumbericSequence = ...,
        std: NumbericSequence = ...,
        data_format: DataLayoutImage = ...,
        to_rgb: bool = ...,
        keys: Optional[Sequence[str]] = ...,
    ) -> None: ...

class Transpose(BaseTransform):
    order: Any = ...
    def __init__(
        self,
        order: Sequence[int] = ...,
        keys: Optional[Sequence[str]] = ...,
    ) -> None: ...

class BrightnessTransform(BaseTransform):
    value: Any = ...
    def __init__(self, value: float, keys: Optional[Sequence[str]] = ...) -> None: ...

class ContrastTransform(BaseTransform):
    value: Any = ...
    def __init__(self, value: float, keys: Optional[Sequence[str]] = ...) -> None: ...

class SaturationTransform(BaseTransform):
    value: Any = ...
    def __init__(self, value: float, keys: Optional[Sequence[str]] = ...) -> None: ...

class HueTransform(BaseTransform):
    value: Any = ...
    def __init__(self, value: float, keys: Optional[Sequence[str]] = ...) -> None: ...

class ColorJitter(BaseTransform):
    brightness: Any = ...
    contrast: Any = ...
    saturation: Any = ...
    hue: Any = ...
    def __init__(
        self,
        brightness: float = ...,
        contrast: float = ...,
        saturation: float = ...,
        hue: float = ...,
        keys: Optional[Sequence[str]] = ...,
    ) -> None: ...

class RandomCrop(BaseTransform):
    size: Any = ...
    padding: Any = ...
    pad_if_needed: Any = ...
    fill: Any = ...
    padding_mode: Any = ...
    def __init__(
        self,
        size: int | IntSequence,
        padding: Optional[int | IntSequence] = ...,
        pad_if_needed: bool = ...,
        fill: int = ...,
        padding_mode: str = ...,
        keys: Optional[Sequence[str]] = ...,
    ) -> None: ...

class Pad(BaseTransform):
    padding: Any = ...
    fill: Any = ...
    padding_mode: Any = ...
    def __init__(
        self,
        padding: int | IntSequence,
        fill: int | IntSequence = ...,
        padding_mode: str = ...,
        keys: Optional[Sequence[str]] = ...,
    ) -> None: ...

class RandomRotation(BaseTransform):
    degrees: Any = ...
    interpolation: Any = ...
    expand: Any = ...
    center: Any = ...
    fill: Any = ...
    def __init__(
        self,
        degrees: float | Sequence[float],
        interpolation: _InterpolationPil | _InterpolationCv2 = ...,
        expand: bool = ...,
        center: Optional[tuple[int, int]] = ...,
        fill: float = ...,
        keys: Optional[Sequence[str]] = ...,
    ) -> None: ...

class Grayscale(BaseTransform):
    num_output_channels: Any = ...
    def __init__(
        self,
        num_output_channels: Literal[1, 3] = ...,
        keys: Optional[Sequence[str]] = ...,
    ) -> None: ...



from __future__ import annotations

from typing import Any, Optional

from paddle.nn import Layer, LayerList
from typing_extensions import Literal

from ..._typing import DTypeLike, ShapeLike, Tensor
from ...framework import ParamAttr

RNNDirection = Literal["forward", "bidirect", "bidirectional"]

class RNNCellBase(Layer):
    shape: Any = ...
    def get_initial_states(
        self,
        batch_ref: Tensor,
        shape: Optional[ShapeLike] = ...,
        dtype: Optional[DTypeLike] = ...,
        init_value: float = ...,
        batch_dim_idx: int = ...,
    ) -> Tensor | list[Tensor]: ...
    @property
    def state_shape(self) -> None: ...
    @property
    def state_dtype(self) -> None: ...

class SimpleRNNCell(RNNCellBase):
    weight_ih: Any = ...
    weight_hh: Any = ...
    bias_ih: Any = ...
    bias_hh: Any = ...
    input_size: Any = ...
    hidden_size: Any = ...
    activation: Any = ...
    def __init__(
        self,
        input_size: int,
        hidden_size: int,
        activation: Literal["relu", "tanh"] = ...,
        weight_ih_attr: Optional[ParamAttr] = ...,
        weight_hh_attr: Optional[ParamAttr] = ...,
        bias_ih_attr: Optional[ParamAttr] = ...,
        bias_hh_attr: Optional[ParamAttr] = ...,
        name: Optional[str] = ...,
    ) -> None: ...
    def forward(
        self,
        inputs: Tensor,
        states: Optional[Tensor] = ...,
    ) -> tuple[Tensor, Tensor]: ...
    __call__ = forward
    @property
    def state_shape(self) -> tuple[int]: ...

class LSTMCell(RNNCellBase):
    weight_ih: Any = ...
    weight_hh: Any = ...
    bias_ih: Any = ...
    bias_hh: Any = ...
    hidden_size: Any = ...
    input_size: Any = ...
    def __init__(
        self,
        input_size: int,
        hidden_size: int,
        weight_ih_attr: Optional[ParamAttr] = ...,
        weight_hh_attr: Optional[ParamAttr] = ...,
        bias_ih_attr: Optional[ParamAttr] = ...,
        bias_hh_attr: Optional[ParamAttr] = ...,
        name: Optional[str] = ...,
    ) -> None: ...
    def forward(
        self,
        inputs: Tensor,
        states: Optional[Tensor] = ...,
    ) -> tuple[Tensor, tuple[Tensor, Tensor]]: ...
    __call__ = forward
    @property
    def state_shape(self) -> tuple[tuple[int], tuple[int]]: ...

class GRUCell(RNNCellBase):
    weight_ih: Any = ...
    weight_hh: Any = ...
    bias_ih: Any = ...
    bias_hh: Any = ...
    hidden_size: Any = ...
    input_size: Any = ...
    def __init__(
        self,
        input_size: int,
        hidden_size: int,
        weight_ih_attr: Optional[ParamAttr] = ...,
        weight_hh_attr: Optional[ParamAttr] = ...,
        bias_ih_attr: Optional[ParamAttr] = ...,
        bias_hh_attr: Optional[ParamAttr] = ...,
        name: Optional[str] = ...,
    ) -> None: ...
    def forward(self, inputs: Tensor, states: Optional[Tensor] = ...) -> tuple[Tensor, Tensor]: ...
    __call__ = forward
    @property
    def state_shape(self) -> tuple[int]: ...

class RNN(Layer):
    cell: Any = ...
    is_reverse: Any = ...
    time_major: Any = ...
    def __init__(
        self,
        cell: RNNCellBase,
        is_reverse: bool = ...,
        time_major: bool = ...,
    ) -> None: ...
    def forward(
        self,
        inputs: Tensor,
        initial_states: Optional[Tensor] = ...,
        sequence_length: Optional[Tensor] = ...,
        **kwargs: Any,
    ) -> tuple[Tensor, Tensor]: ...
    __call__ = forward

class BiRNN(Layer):
    cell_fw: Any = ...
    cell_bw: Any = ...
    time_major: Any = ...
    def __init__(
        self,
        cell_fw: RNNCellBase,
        cell_bw: RNNCellBase,
        time_major: bool = ...,
    ) -> None: ...
    def forward(
        self,
        inputs: Tensor,
        initial_states: Optional[Tensor] = ...,
        sequence_length: Optional[Tensor] = ...,
        **kwargs: Any,
    ) -> tuple[Tensor, Tensor]: ...
    __call__ = forward

class RNNBase(LayerList):
    mode: Any = ...
    input_size: Any = ...
    hidden_size: Any = ...
    dropout: Any = ...
    num_directions: Any = ...
    time_major: Any = ...
    num_layers: Any = ...
    state_components: Any = ...
    could_use_cudnn: bool = ...
    def __init__(
        self,
        mode: Literal["RNN_TANH", "RNN_RELU", "LSTM", "GRU"],
        input_size: int,
        hidden_size: int,
        num_layers: int = ...,
        direction: RNNDirection = ...,
        time_major: bool = ...,
        dropout: float = ...,
        weight_ih_attr: Optional[ParamAttr] = ...,
        weight_hh_attr: Optional[ParamAttr] = ...,
        bias_ih_attr: Optional[ParamAttr] = ...,
        bias_hh_attr: Optional[ParamAttr] = ...,
    ) -> None: ...
    def flatten_parameters(self) -> None: ...
    def forward(
        self,
        inputs: Tensor,
        initial_states: Optional[Tensor] = ...,
        sequence_length: Optional[Tensor] = ...,
    ) -> tuple[Tensor, Tensor]: ...
    __call__ = forward

class SimpleRNN(RNNBase):
    activation: Any = ...
    def __init__(
        self,
        input_size: int,
        hidden_size: int,
        num_layers: int = ...,
        direction: RNNDirection = ...,
        time_major: bool = ...,
        dropout: float = ...,
        activation: str = ...,
        weight_ih_attr: Optional[ParamAttr] = ...,
        weight_hh_attr: Optional[ParamAttr] = ...,
        bias_ih_attr: Optional[ParamAttr] = ...,
        bias_hh_attr: Optional[ParamAttr] = ...,
        name: Optional[str] = ...,
    ) -> None: ...

class LSTM(RNNBase):
    def __init__(
        self,
        input_size: int,
        hidden_size: int,
        num_layers: int = ...,
        direction: RNNDirection = ...,
        time_major: bool = ...,
        dropout: float = ...,
        weight_ih_attr: Optional[ParamAttr] = ...,
        weight_hh_attr: Optional[ParamAttr] = ...,
        bias_ih_attr: Optional[ParamAttr] = ...,
        bias_hh_attr: Optional[ParamAttr] = ...,
        name: Optional[str] = ...,
    ) -> None: ...

class GRU(RNNBase):
    def __init__(
        self,
        input_size: int,
        hidden_size: int,
        num_layers: int = ...,
        direction: RNNDirection = ...,
        time_major: bool = ...,
        dropout: float = ...,
        weight_ih_attr: Optional[ParamAttr] = ...,
        weight_hh_attr: Optional[ParamAttr] = ...,
        bias_ih_attr: Optional[ParamAttr] = ...,
        bias_hh_attr: Optional[ParamAttr] = ...,
        name: Optional[str] = ...,
    ) -> None: ...



from __future__ import annotations

from typing import Any, Optional

from typing_extensions import Literal

from ..._typing import (
    DataLayout1D,
    DataLayout2D,
    DataLayout3D,
    IntSequence,
    ShapeLike,
    Tensor,
)
from ...framework import ParamAttr
from .. import Layer

PaddingSizeStr = Literal["valid", "same"]
PaddingMode = Literal["zeros", "reflect", "replicate", "circular"]

class _ConvNd(Layer):
    output_padding: Any = ...
    weight: Any = ...
    bias: Any = ...
    def __init__(
        self,
        in_channels: Any,
        out_channels: Any,
        kernel_size: Any,
        transposed: Any,
        dims: Any,
        stride: int = ...,
        padding: int = ...,
        padding_mode: str = ...,
        output_padding: int = ...,
        dilation: int = ...,
        groups: int = ...,
        weight_attr: Optional[Any] = ...,
        bias_attr: Optional[Any] = ...,
        data_format: str = ...,
    ) -> None: ...
    def extra_repr(self) -> str: ...

class Conv1D(_ConvNd):
    def __init__(
        self,
        in_channels: int,
        out_channels: int,
        kernel_size: int | IntSequence,
        stride: int | IntSequence = ...,
        padding: int | IntSequence | PaddingSizeStr = ...,
        dilation: int = ...,
        groups: int = ...,
        padding_mode: str = ...,
        weight_attr: Optional[ParamAttr] = ...,
        bias_attr: Optional[ParamAttr | bool] = ...,
        data_format: DataLayout1D = ...,
    ) -> None: ...
    def forward(self, x: Tensor) -> Tensor: ...
    __call__ = forward

class Conv1DTranspose(_ConvNd):
    def __init__(
        self,
        in_channels: int,
        out_channels: int,
        kernel_size: int | IntSequence,
        stride: int | IntSequence = ...,
        padding: int | IntSequence | PaddingSizeStr = ...,
        output_padding: int | IntSequence = ...,
        groups: int = ...,
        dilation: int = ...,
        weight_attr: Optional[ParamAttr] = ...,
        bias_attr: Optional[ParamAttr | bool] = ...,
        data_format: str = ...,
    ) -> None: ...
    def forward(self, x: Tensor, output_size: Optional[ShapeLike] = ...) -> Tensor: ...
    __call__ = forward

class Conv2D(_ConvNd):
    def __init__(
        self,
        in_channels: int,
        out_channels: int,
        kernel_size: int | IntSequence,
        stride: int | IntSequence = ...,
        padding: int | IntSequence | PaddingSizeStr = ...,
        dilation: int | IntSequence = ...,
        groups: int = ...,
        padding_mode: PaddingMode = ...,
        weight_attr: Optional[ParamAttr] = ...,
        bias_attr: Optional[ParamAttr | bool] = ...,
        data_format: DataLayout2D = ...,
    ) -> None: ...
    def forward(self, x: Tensor) -> Tensor: ...
    __call__ = forward

class Conv2DTranspose(_ConvNd):
    def __init__(
        self,
        in_channels: int,
        out_channels: int,
        kernel_size: int | IntSequence,
        stride: int | IntSequence = ...,
        padding: int | IntSequence | PaddingSizeStr = ...,
        output_padding: int | IntSequence = ...,
        dilation: int | IntSequence = ...,
        groups: int = ...,
        weight_attr: Optional[ParamAttr] = ...,
        bias_attr: Optional[ParamAttr | bool] = ...,
        data_format: DataLayout2D = ...,
    ) -> None: ...
    def forward(self, x: Tensor, output_size: Optional[ShapeLike] = ...) -> Tensor: ...
    __call__ = forward

class Conv3D(_ConvNd):
    def __init__(
        self,
        in_channels: int,
        out_channels: int,
        kernel_size: int | IntSequence,
        stride: int | IntSequence = ...,
        padding: int | IntSequence | PaddingSizeStr = ...,
        dilation: int | IntSequence = ...,
        groups: int = ...,
        padding_mode: PaddingMode = ...,
        weight_attr: Optional[ParamAttr] = ...,
        bias_attr: Optional[ParamAttr | bool] = ...,
        data_format: DataLayout3D = ...,
    ) -> None: ...
    def forward(self, x: Tensor) -> Tensor: ...
    __call__ = forward

class Conv3DTranspose(_ConvNd):
    def __init__(
        self,
        in_channels: int,
        out_channels: int,
        kernel_size: int | IntSequence,
        stride: int | IntSequence = ...,
        padding: int | IntSequence | PaddingSizeStr = ...,
        output_padding: int | IntSequence = ...,
        dilation: int | IntSequence = ...,
        groups: int = ...,
        weight_attr: Optional[ParamAttr] = ...,
        bias_attr: Optional[ParamAttr | bool] = ...,
        data_format: DataLayout3D = ...,
    ) -> None: ...
    def forward(self, x: Tensor, output_size: Optional[ShapeLike] = ...) -> Tensor: ...
    __call__ = forward



from __future__ import annotations

from typing_extensions import Literal

DataLayout0D = Literal["NC"]
DataLayout1D = Literal["NCL", "NLC"]
DataLayout2D = Literal["NCHW", "NHCW"]
DataLayout3D = Literal["NCDHW", "NDHWC"]
DataLayoutND = DataLayout0D | DataLayout1D | DataLayout2D | DataLayout3D

DataLayout1DVariant = Literal["NCW", "NWC"]
DataLayoutImage = Literal["HWC", "CHW"]



from __future__ import annotations

from typing import Any, Iterable, Optional

import numpy as np
import numpy.typing as npt
from paddle import nn
from typing_extensions import Literal

from .._typing import DTypeLike, DynamicShapeLike, Tensor
from ..callbacks import Callback
from ..io import DataLoader, Dataset
from ..metric import Metric
from ..optimizer import Optimizer
from ..static import InputSpec
from .model_summary import ModelSummary

class Model:
    mode: str = ...
    network: nn.Layer = ...
    stop_training: bool = ...
    def __init__(
        self,
        network: nn.Layer,
        inputs: Optional[InputSpec | list[int] | tuple[int, ...] | dict[str, InputSpec]] = ...,
        labels: Optional[InputSpec | list[int] | tuple[int, ...] | dict[str, InputSpec]] = ...,
    ) -> None: ...
    def train_batch(
        self,
        inputs: npt.NDArray[Any] | Tensor | list[Tensor],
        labels: Optional[npt.NDArray[Any] | Tensor | list[Tensor]] = ...,
        update: bool = ...,
    ) -> list[npt.NDArray[Any]] | tuple[list[npt.NDArray[Any]], list[npt.NDArray[Any]]]: ...
    def eval_batch(
        self,
        inputs: npt.NDArray[Any] | Tensor | list[Tensor],
        labels: Optional[npt.NDArray[Any] | Tensor | list[Tensor]] = ...,
    ) -> list[npt.NDArray[Any]] | tuple[list[npt.NDArray[Any]], list[npt.NDArray[Any]]]: ...
    def predict_batch(
        self,
        inputs: npt.NDArray[Any] | Tensor | list[Tensor],
    ) -> list[npt.NDArray[Any]]: ...
    def save(self, path: str, training: bool = ...) -> None: ...
    def load(self, path: str, skip_mismatch: bool = ..., reset_optimizer: bool = ...) -> None: ...
    def parameters(self, *args: Any, **kwargs: Any) -> list[Tensor]: ...
    def prepare(
        self,
        optimizer: Optional[Optimizer] = ...,
        loss: Optional[nn.Layer] = ...,
        metrics: Optional[Metric | list[Metric]] = ...,
        amp_configs: Optional[Literal["O0", "O1", "O2"] | dict[str, Any]] = ...,
    ) -> None: ...
    def fit(
        self,
        train_data: Optional[Iterable[Tensor] | Dataset | DataLoader] = ...,
        eval_data: Optional[Iterable[Tensor] | Dataset | DataLoader] = ...,
        batch_size: int = ...,
        epochs: int = ...,
        eval_freq: int = ...,
        log_freq: int = ...,
        save_dir: Optional[str] = ...,
        save_freq: int = ...,
        verbose: int = ...,
        drop_last: bool = ...,
        shuffle: bool = ...,
        num_workers: int = ...,
        callbacks: Optional[Callback | list[Callback]] = ...,
        accumulate_grad_batches: int = ...,
        num_iters: Optional[int] = ...,
    ) -> None: ...
    def evaluate(
        self,
        eval_data: Optional[Iterable[Tensor] | Dataset | DataLoader],
        batch_size: int = ...,
        log_freq: int = ...,
        verbose: int = ...,
        num_workers: int = ...,
        callbacks: Optional[Callback | list[Callback]] = ...,
        num_iters: Optional[int] = ...,
    ) -> dict[str, npt.NDArray[Any]]: ...
    def predict(
        self,
        test_data: Optional[Iterable[Tensor] | Dataset | DataLoader],
        batch_size: int = ...,
        num_workers: int = ...,
        stack_outputs: bool = ...,
        verbose: int = ...,
        callbacks: Optional[Callback | list[Callback]] = ...,
    ) -> Tensor: ...  # TODO: multiple output model should return list of tensors
    def summary(
        self,
        input_size: Optional[DynamicShapeLike | InputSpec | list[DynamicShapeLike | InputSpec]] = ...,
        dtype: Optional[DTypeLike] = ...,
    ) -> ModelSummary: ...



from __future__ import annotations

from typing import Any, Optional

import numpy as np
from typing_extensions import Literal

class dtype:
    def __init__(self, arg0: int) -> None: ...

uint8: dtype
int8: dtype
int16: dtype
int32: dtype
int64: dtype
float32: dtype
float64: dtype
float16: dtype
bfloat16: dtype
complex64: dtype
complex128: dtype
bool: dtype

_DTypeString = Literal[
    "uint8",
    "int8",
    "int16",
    "int32",
    "int64",
    "float32",
    "float64",
    "float16",
    "bfloat16",
    "complex64",
    "complex128",
    "bool",
]

_DTypeNumpy = (
    type[np.uint8]
    | type[np.int8]
    | type[np.int16]
    | type[np.int32]
    | type[np.int64]
    | type[np.float32]
    | type[np.float64]
    | type[np.float16]
    | type[np.complex64]
    | type[np.complex128]
    | type[np.bool_]
    | np.dtype[Any]
)

DTypeLike = Optional[dtype | _DTypeNumpy | _DTypeString]



from __future__ import annotations

# Basic
from .basic import IntSequence as IntSequence
from .basic import NestedNumbericSequence as NestedNumbericSequence
from .basic import Numberic as Numberic
from .basic import NumbericSequence as NumbericSequence

# Device
from .device import CPUPlace as CPUPlace
from .device import CUDAPlace as CUDAPlace
from .device import CustomPlace as CustomPlace
from .device import IPUPlace as IPUPlace
from .device import MLUPlace as MLUPlace
from .device import NPUPlace as NPUPlace
from .device import Place as Place
from .device import PlaceLike as PlaceLike
from .device import XPUPlace as XPUPlace

# DType
from .dtype import DTypeLike as DTypeLike
from .dtype import bfloat16 as bfloat16
from .dtype import bool as bool
from .dtype import complex64 as complex64
from .dtype import complex128 as complex128
from .dtype import dtype as dtype
from .dtype import float16 as float16
from .dtype import float32 as float32
from .dtype import float64 as float64
from .dtype import int8 as int8
from .dtype import int16 as int16
from .dtype import int32 as int32
from .dtype import int64 as int64
from .dtype import uint8 as uint8

# DataLayout
from .layout import DataLayout0D as DataLayout0D
from .layout import DataLayout1D as DataLayout1D
from .layout import DataLayout1DVariant as DataLayout1DVariant
from .layout import DataLayout2D as DataLayout2D
from .layout import DataLayout3D as DataLayout3D
from .layout import DataLayoutImage as DataLayoutImage
from .layout import DataLayoutND as DataLayoutND

# Shape
from .shape import DynamicShapeLike as DynamicShapeLike
from .shape import ShapeLike as ShapeLike

# Tensor
from .tensor import Tensor as Tensor
from .tensor import TensorLike as TensorLike



from __future__ import annotations

from typing import Any, Iterator, Optional, OrderedDict

from .. import Layer

class LayerDict(Layer):
    def __init__(
        self, sublayers: Optional[OrderedDict[str, Layer] | list[tuple[str, Layer]] | LayerDict] = ...
    ) -> None: ...
    def __getitem__(self, key: str) -> Layer: ...
    def __setitem__(self, key: str, sublayer: Layer) -> None: ...
    def __delitem__(self, key: Any) -> None: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Any: ...
    def __contains__(self, key: Layer) -> bool: ...
    def clear(self) -> None: ...
    def pop(self, key: Any) -> Layer: ...
    def keys(self) -> Iterator[str]: ...
    def items(self) -> Iterator[tuple[str, Layer]]: ...
    def values(self) -> Iterator[Layer]: ...
    def update(self, sublayers: Any) -> None: ...



from __future__ import annotations

from typing import Any, NamedTuple, Optional, Sequence

from ... import tensor as tensor
from ..._typing import Tensor
from ...framework import ParamAttr
from .. import Layer as Layer
from .. import LayerList as LayerList
from .common import Dropout as Dropout
from .common import Linear as Linear
from .norm import LayerNorm

class MultiHeadAttention(Layer):
    class Cache(NamedTuple):
        k: Tensor
        v: Tensor

    class StaticCache(NamedTuple):
        k: Tensor
        v: Tensor
    embed_dim: Any = ...
    kdim: Any = ...
    vdim: Any = ...
    num_heads: Any = ...
    dropout: Any = ...
    need_weights: Any = ...
    head_dim: Any = ...
    q_proj: Any = ...
    k_proj: Any = ...
    v_proj: Any = ...
    out_proj: Any = ...
    def __init__(
        self,
        embed_dim: int,
        num_heads: int,
        dropout: float = ...,
        kdim: Optional[int] = ...,
        vdim: Optional[int] = ...,
        need_weights: bool = ...,
        weight_attr: Optional[ParamAttr] = ...,
        bias_attr: Optional[ParamAttr] = ...,
    ) -> None: ...
    def forward(
        self,
        query: Tensor,
        key: Optional[Tensor] = ...,
        value: Optional[Tensor] = ...,
        attn_mask: Optional[Tensor] = ...,
        cache: Optional[MultiHeadAttention.Cache | MultiHeadAttention.StaticCache] = ...,
    ) -> Tensor | tuple[Tensor, Tensor] | tuple[Tensor, Tensor, Tensor]: ...
    __call__ = forward

class TransformerEncoderLayer(Layer):
    normalize_before: Any = ...
    self_attn: Any = ...
    linear1: Any = ...
    dropout: Any = ...
    linear2: Any = ...
    norm1: Any = ...
    norm2: Any = ...
    dropout1: Any = ...
    dropout2: Any = ...
    activation: Any = ...
    def __init__(
        self,
        d_model: int,
        nhead: int,
        dim_feedforward: int,
        dropout: float = ...,
        activation: str = ...,
        attn_dropout: Optional[float] = ...,
        act_dropout: Optional[float] = ...,
        normalize_before: bool = ...,
        weight_attr: Optional[ParamAttr | Sequence[ParamAttr]] = ...,
        bias_attr: Optional[ParamAttr | Sequence[ParamAttr] | bool] = ...,
    ) -> None: ...
    def forward(
        self,
        src: Tensor,
        src_mask: Optional[Tensor] = ...,
        cache: Optional[Tensor] = ...,
    ) -> Tensor | tuple[Tensor, Tensor]: ...
    __call__ = forward

class TransformerEncoder(Layer):
    layers: Any = ...
    num_layers: Any = ...
    norm: Any = ...
    def __init__(
        self,
        encoder_layer: Layer,
        num_layers: int,
        norm: Optional[LayerNorm] = ...,
    ) -> None: ...
    def forward(
        self,
        src: Tensor,
        src_mask: Optional[Tensor] = ...,
        cache: Optional[list[MultiHeadAttention.Cache | MultiHeadAttention.StaticCache]] = ...,
    ) -> Tensor | tuple[Tensor, Tensor]: ...
    __call__ = forward

class TransformerDecoderLayer(Layer):
    normalize_before: Any = ...
    self_attn: Any = ...
    cross_attn: Any = ...
    linear1: Any = ...
    dropout: Any = ...
    linear2: Any = ...
    norm1: Any = ...
    norm2: Any = ...
    norm3: Any = ...
    dropout1: Any = ...
    dropout2: Any = ...
    dropout3: Any = ...
    activation: Any = ...
    def __init__(
        self,
        d_model: int,
        nhead: int,
        dim_feedforward: int,
        dropout: float = ...,
        activation: str = ...,
        attn_dropout: Optional[float] = ...,
        act_dropout: Optional[float] = ...,
        normalize_before: bool = ...,
        weight_attr: Optional[ParamAttr | Sequence[ParamAttr]] = ...,
        bias_attr: Optional[ParamAttr | Sequence[ParamAttr] | bool] = ...,
    ) -> None: ...
    def forward(
        self,
        tgt: Tensor,
        memory: Tensor,
        tgt_mask: Optional[Tensor] = ...,
        memory_mask: Optional[Tensor] = ...,
        cache: Optional[tuple[MultiHeadAttention.Cache, ...] | tuple[MultiHeadAttention.StaticCache, ...]] = ...,
    ) -> Tensor | tuple[Tensor, tuple[MultiHeadAttention.Cache, MultiHeadAttention.StaticCache]]: ...
    __call__ = forward

class TransformerDecoder(Layer):
    layers: Any = ...
    num_layers: Any = ...
    norm: Any = ...
    def __init__(
        self,
        decoder_layer: Layer,
        num_layers: int,
        norm: Optional[LayerNorm] = ...,
    ) -> None: ...
    def forward(
        self,
        tgt: Tensor,
        memory: Tensor,
        tgt_mask: Optional[Tensor] = ...,
        memory_mask: Optional[Tensor] = ...,
        cache: Optional[list[MultiHeadAttention.Cache] | list[MultiHeadAttention.StaticCache]] = ...,
    ) -> Tensor | tuple[Tensor, Tensor]: ...
    __call__ = forward

class Transformer(Layer):
    encoder: Any = ...
    decoder: Any = ...
    d_model: Any = ...
    nhead: Any = ...
    def __init__(
        self,
        d_model: int = ...,
        nhead: int = ...,
        num_encoder_layers: int = ...,
        num_decoder_layers: int = ...,
        dim_feedforward: int = ...,
        dropout: float = ...,
        activation: str = ...,
        attn_dropout: Optional[float] = ...,
        act_dropout: Optional[float] = ...,
        normalize_before: bool = ...,
        weight_attr: Optional[ParamAttr | Sequence[ParamAttr]] = ...,
        bias_attr: Optional[ParamAttr | Sequence[ParamAttr] | bool] = ...,
        custom_encoder: Optional[Layer] = ...,
        custom_decoder: Optional[Layer] = ...,
    ) -> None: ...
    def forward(
        self,
        src: Tensor,
        tgt: Tensor,
        src_mask: Optional[Tensor] = ...,
        tgt_mask: Optional[Tensor] = ...,
        memory_mask: Optional[Tensor] = ...,
    ) -> Tensor: ...
    __call__ = forward



from __future__ import annotations

from typing import Optional

from .._typing import Tensor
from ..fluid.clip import GradientClipBase
from ..fluid.regularizer import WeightDecayRegularizer
from .lr import LRScheduler
from .optimizer import Optimizer, ParameterConfig

class SGD(Optimizer):
    type: str = ...
    def __init__(
        self,
        learning_rate: float | LRScheduler = ...,
        parameters: Optional[list[Tensor] | list[ParameterConfig]] = ...,
        weight_decay: Optional[float | WeightDecayRegularizer] = ...,
        grad_clip: Optional[GradientClipBase] = ...,
        multi_precision: bool = ...,
        name: Optional[str] = ...,
    ) -> None: ...



from __future__ import annotations

from typing import Any, TypeVar

import numpy as np

Numberic = int | float | complex | np.number[Any]

_T = TypeVar("_T", bound=Numberic)
_SeqLevel1 = tuple[_T, ...] | list[_T]

_TL1 = TypeVar("_TL1", bound=_SeqLevel1[Numberic])
_SeqLevel2 = tuple[_TL1, ...] | list[_TL1]

_TL2 = TypeVar("_TL2", bound=_SeqLevel2[_SeqLevel1[Numberic]])
_SeqLevel3 = tuple[_TL2, ...] | list[_TL2]

_TL3 = TypeVar("_TL3", bound=_SeqLevel3[_SeqLevel2[_SeqLevel1[Numberic]]])
_SeqLevel4 = tuple[_TL3, ...] | list[_TL3]

_TL4 = TypeVar("_TL4", bound=_SeqLevel4[_SeqLevel3[_SeqLevel2[_SeqLevel1[Numberic]]]])
_SeqLevel5 = tuple[_TL4, ...] | list[_TL4]

_TL5 = TypeVar("_TL5", bound=_SeqLevel5[_SeqLevel4[_SeqLevel3[_SeqLevel2[_SeqLevel1[Numberic]]]]])
_SeqLevel6 = tuple[_TL5, ...] | list[_TL5]

IntSequence = _SeqLevel1[int]
NumbericSequence = _SeqLevel1[Numberic]
NestedNumbericSequence = (
    Numberic
    | _SeqLevel1[Numberic]
    | _SeqLevel2[_SeqLevel1[Numberic]]
    | _SeqLevel3[_SeqLevel2[_SeqLevel1[Numberic]]]
    | _SeqLevel4[_SeqLevel3[_SeqLevel2[_SeqLevel1[Numberic]]]]
    | _SeqLevel5[_SeqLevel4[_SeqLevel3[_SeqLevel2[_SeqLevel1[Numberic]]]]]
    | _SeqLevel6[_SeqLevel5[_SeqLevel4[_SeqLevel3[_SeqLevel2[_SeqLevel1[Numberic]]]]]]
)



from __future__ import annotations

from typing import Any, Optional

from paddle import in_dynamic_mode as in_dynamic_mode

from ..._typing import Tensor
from ...fluid.data_feeder import check_type as check_type
from ...fluid.data_feeder import check_variable_and_dtype as check_variable_and_dtype
from ...fluid.layer_helper import LayerHelper as LayerHelper
from .. import Layer as Layer

class PairwiseDistance(Layer):
    p: Any = ...
    epsilon: Any = ...
    keepdim: Any = ...
    name: Any = ...
    def __init__(
        self,
        p: float = ...,
        epsilon: float = ...,
        keepdim: bool = ...,
        name: Optional[str] = ...,
    ) -> None: ...
    def forward(self, x: Tensor, y: Tensor) -> Tensor: ...
    __call__ = forward



from __future__ import annotations

from typing import Optional

from ..._typing import DataLayout2D, Tensor
from .. import Layer as Layer
from .. import functional as functional

class PixelShuffle(Layer):
    def __init__(
        self,
        upscale_factor: int,
        data_format: DataLayout2D = ...,
        name: Optional[str] = ...,
    ) -> None: ...
    def forward(self, x: Tensor) -> Tensor: ...
    __call__ = forward



from __future__ import annotations

from typing import Optional

from ...fluid.initializer import MSRAInitializer

class KaimingNormal(MSRAInitializer):
    def __init__(
        self,
        fan_in: Optional[float] = ...,
        negative_slope: float = ...,
        nonlinearity: str = ...,
    ) -> None: ...

class KaimingUniform(MSRAInitializer):
    def __init__(
        self,
        fan_in: Optional[float] = ...,
        negative_slope: float = ...,
        nonlinearity: str = ...,
    ) -> None: ...



from __future__ import annotations

from typing import Any, Callable, Optional

import numpy as np
import numpy.typing as npt
from typing_extensions import Literal, TypedDict

from .._typing import Tensor

class LRStateDict(TypedDict):
    last_epoch: int
    last_lr: float

class LinearWarmupLRStateDict(LRStateDict):
    LinearWarmup_LR: LRStateDict

class ReduceOnPlateauLRStateDict(LRStateDict):
    cooldown_counter: int
    best: int
    num_bad_epochs: int

class LRScheduler:
    base_lr: Any = ...
    last_lr: Any = ...
    last_epoch: Any = ...
    verbose: Any = ...
    def __init__(
        self,
        learning_rate: float = ...,
        last_epoch: int = ...,
        verbose: bool = ...,
    ) -> None: ...
    def __call__(self) -> float: ...
    def step(self, epoch: Optional[int] = ...) -> None: ...
    def state_dict(self) -> LRStateDict: ...
    keys: str = ...
    def state_keys(self) -> None: ...
    def set_state_dict(self, state_dict: LRStateDict) -> None: ...
    set_dict = set_state_dict
    def get_lr(self) -> float: ...

class NoamDecay(LRScheduler):
    d_model: Any = ...
    warmup_steps: Any = ...
    def __init__(
        self,
        d_model: int,
        warmup_steps: int,
        learning_rate: float = ...,
        last_epoch: int = ...,
        verbose: bool = ...,
    ) -> None: ...
    def get_lr(self) -> float: ...

class PiecewiseDecay(LRScheduler):
    boundaries: Any = ...
    values: Any = ...
    def __init__(
        self,
        boundaries: list[int] | tuple[int, ...],
        values: list[float] | tuple[float, ...],
        last_epoch: int = ...,
        verbose: bool = ...,
    ) -> None: ...
    def get_lr(self) -> float: ...

class NaturalExpDecay(LRScheduler):
    gamma: Any = ...
    def __init__(
        self,
        learning_rate: float,
        gamma: float,
        last_epoch: int = ...,
        verbose: bool = ...,
    ) -> None: ...
    def get_lr(self) -> float: ...

class InverseTimeDecay(LRScheduler):
    gamma: Any = ...
    def __init__(
        self,
        learning_rate: float,
        gamma: float,
        last_epoch: int = ...,
        verbose: bool = ...,
    ) -> None: ...
    def get_lr(self) -> float: ...

class PolynomialDecay(LRScheduler):
    decay_steps: Any = ...
    end_lr: Any = ...
    power: Any = ...
    cycle: Any = ...
    def __init__(
        self,
        learning_rate: float,
        decay_steps: int,
        end_lr: float = ...,
        power: float = ...,
        cycle: bool = ...,
        last_epoch: int = ...,
        verbose: bool = ...,
    ) -> None: ...
    def get_lr(self) -> float: ...

class LinearWarmup(LRScheduler):
    learning_rate: Any = ...
    warmup_steps: Any = ...
    start_lr: Any = ...
    end_lr: Any = ...
    def __init__(
        self,
        learning_rate: float | LRScheduler,
        warmup_steps: int,
        start_lr: float,
        end_lr: float,
        last_epoch: int = ...,
        verbose: bool = ...,
    ) -> None: ...
    def state_dict(self) -> LinearWarmupLRStateDict | LRStateDict: ...
    def set_state_dict(
        self,
        state_dict: LinearWarmupLRStateDict | LRStateDict,
    ) -> None: ...
    def get_lr(self) -> float: ...

class ExponentialDecay(LRScheduler):
    gamma: Any = ...
    def __init__(
        self,
        learning_rate: float,
        gamma: float,
        last_epoch: int = ...,
        verbose: bool = ...,
    ) -> None: ...
    def get_lr(self) -> float: ...

class MultiStepDecay(LRScheduler):
    milestones: Any = ...
    gamma: Any = ...
    def __init__(
        self,
        learning_rate: float,
        milestones: list[int] | tuple[int, ...],
        gamma: float = ...,
        last_epoch: int = ...,
        verbose: bool = ...,
    ) -> None: ...
    def get_lr(self) -> float: ...

class StepDecay(LRScheduler):
    step_size: Any = ...
    gamma: Any = ...
    def __init__(
        self,
        learning_rate: float,
        step_size: int,
        gamma: float = ...,
        last_epoch: int = ...,
        verbose: bool = ...,
    ) -> None: ...
    def get_lr(self) -> float: ...

class LambdaDecay(LRScheduler):
    lr_lambda: Any = ...
    def __init__(
        self,
        learning_rate: float,
        lr_lambda: Callable[[float], float],
        last_epoch: int = ...,
        verbose: bool = ...,
    ) -> None: ...
    def get_lr(self) -> float: ...

class ReduceOnPlateau(LRScheduler):
    mode: Any = ...
    factor: Any = ...
    threshold_mode: Any = ...
    patience: Any = ...
    threshold: Any = ...
    cooldown: Any = ...
    min_lr: Any = ...
    epsilon: Any = ...
    cooldown_counter: int = ...
    best: Any = ...
    num_bad_epochs: int = ...
    base_lr: Any = ...
    last_lr: Any = ...
    last_epoch: int = ...
    verbose: Any = ...
    def __init__(
        self,
        learning_rate: float,
        mode: Literal["min", "max"] = ...,
        factor: float = ...,
        patience: int = ...,
        threshold: float = ...,
        threshold_mode: Literal["rel", "abs"] = ...,
        cooldown: int = ...,
        min_lr: float = ...,
        epsilon: float = ...,
        verbose: bool = ...,
    ) -> None: ...
    keys: Any = ...
    def state_keys(self) -> None: ...
    def step(
        self,
        metrics: Tensor | npt.NDArray[Any] | float,
        epoch: Optional[int] = ...,
    ) -> None: ...

class CosineAnnealingDecay(LRScheduler):
    T_max: Any = ...
    eta_min: Any = ...
    def __init__(
        self,
        learning_rate: float,
        T_max: int,
        eta_min: float | int = ...,
        last_epoch: int = ...,
        verbose: bool = ...,
    ) -> None: ...
    def get_lr(self) -> float: ...

class MultiplicativeDecay(LRScheduler):
    lr_lambda: Any = ...
    def __init__(
        self,
        learning_rate: float,
        lr_lambda: Callable[[float], float],
        last_epoch: int = ...,
        verbose: bool = ...,
    ) -> None: ...
    def get_lr(self) -> float: ...



from __future__ import annotations

from typing import Any, Optional

from typing_extensions import Literal

from ..._typing import Tensor
from ...framework import ParamAttr
from .. import Layer

Reduction = Literal["none", "sum", "mean"]
ReductionBatchMean = Literal[Reduction, "batchmean"]

class BCEWithLogitsLoss(Layer):
    weight: Any = ...
    reduction: Any = ...
    pos_weight: Any = ...
    name: Any = ...
    def __init__(
        self,
        weight: Optional[Tensor] = ...,
        reduction: Reduction = ...,
        pos_weight: Optional[Any] = ...,
        name: Optional[str] = ...,
    ) -> None: ...
    def forward(self, logit: Tensor, label: Tensor) -> Tensor: ...
    __call__ = forward

class CrossEntropyLoss(Layer):
    weight: Any = ...
    reduction: Any = ...
    ignore_index: Any = ...
    soft_label: Any = ...
    axis: Any = ...
    use_softmax: Any = ...
    name: Any = ...
    def __init__(
        self,
        weight: Optional[Tensor] = ...,
        ignore_index: int = ...,
        reduction: Reduction = ...,
        soft_label: bool = ...,
        axis: int = ...,
        use_softmax: bool = ...,
        name: Optional[str] = ...,
    ) -> None: ...
    def forward(self, input: Tensor, label: Tensor) -> Tensor: ...
    __call__ = forward

class HSigmoidLoss(Layer):
    weight: Any = ...
    bias: Any = ...
    def __init__(
        self,
        feature_size: int,
        num_classes: int,
        weight_attr: Optional[ParamAttr] = ...,
        bias_attr: Optional[ParamAttr | bool] = ...,
        is_custom: bool = ...,
        is_sparse: bool = ...,
        name: Optional[str] = ...,
    ) -> None: ...
    def forward(
        self,
        input: Tensor,
        label: Tensor,
        path_table: Optional[Tensor] = ...,
        path_code: Optional[Tensor] = ...,
    ) -> Tensor: ...
    __call__ = forward

class MSELoss(Layer):
    reduction: Any = ...
    def __init__(self, reduction: Reduction = ...) -> None: ...
    def forward(self, input: Tensor, label: Tensor) -> Tensor: ...
    __call__ = forward

class L1Loss(Layer):
    reduction: Any = ...
    name: Any = ...
    def __init__(self, reduction: Reduction = ..., name: Optional[str] = ...) -> None: ...
    def forward(self, input: Tensor, label: Tensor) -> Tensor: ...
    __call__ = forward

class BCELoss(Layer):
    weight: Any = ...
    reduction: Any = ...
    name: Any = ...
    def __init__(
        self,
        weight: Optional[Tensor] = ...,
        reduction: Reduction = ...,
        name: Optional[str] = ...,
    ) -> None: ...
    def forward(self, input: Tensor, label: Tensor) -> Tensor: ...
    __call__ = forward

class NLLLoss(Layer):
    def __init__(
        self,
        weight: Optional[Tensor] = ...,
        ignore_index: int = ...,
        reduction: Reduction = ...,
        name: Optional[str] = ...,
    ) -> None: ...
    def forward(self, input: Tensor, label: Tensor) -> Tensor: ...
    __call__ = forward

class KLDivLoss(Layer):
    reduction: Any = ...
    def __init__(self, reduction: ReductionBatchMean = ...) -> None: ...
    def forward(self, input: Tensor, label: Tensor) -> Tensor: ...
    __call__ = forward

class MarginRankingLoss(Layer):
    margin: Any = ...
    reduction: Any = ...
    name: Any = ...
    def __init__(
        self,
        margin: float = ...,
        reduction: Reduction = ...,
        name: Optional[str] = ...,
    ) -> None: ...
    def forward(self, input: Tensor, other: Tensor, label: Tensor) -> Tensor: ...
    __call__ = forward

class CTCLoss(Layer):
    blank: Any = ...
    reduction: Any = ...
    def __init__(self, blank: int = ..., reduction: Reduction = ...) -> None: ...
    def forward(
        self,
        log_probs: Tensor,
        labels: Tensor,
        input_lengths: Tensor,
        label_lengths: Tensor,
        norm_by_times: bool = ...,
    ) -> Tensor: ...
    __call__ = forward

class SmoothL1Loss(Layer):
    reduction: Any = ...
    delta: Any = ...
    name: Any = ...
    def __init__(
        self,
        reduction: Reduction = ...,
        delta: float = ...,
        name: Optional[str] = ...,
    ) -> None: ...
    def forward(self, input: Tensor, label: Tensor) -> Tensor: ...
    __call__ = forward

class HingeEmbeddingLoss(Layer):
    margin: Any = ...
    reduction: Any = ...
    name: Any = ...
    def __init__(
        self,
        margin: float = ...,
        reduction: Reduction = ...,
        name: Optional[str] = ...,
    ) -> None: ...
    def forward(self, input: Tensor, label: Tensor) -> Tensor: ...
    __call__ = forward



from __future__ import annotations

from typing import Callable, Optional

from .._typing import Tensor
from ..fluid.clip import GradientClipBase
from ..fluid.regularizer import WeightDecayRegularizer
from .adam import Adam, AdamParameterConfig
from .lr import LRScheduler

class AdamW(Adam):
    type: str = ...
    def __init__(
        self,
        learning_rate: float | LRScheduler = ...,
        beta1: float | Tensor = ...,
        beta2: float | Tensor = ...,
        epsilon: float = ...,
        parameters: Optional[list[Tensor] | list[AdamParameterConfig]] = ...,
        weight_decay: Optional[float | WeightDecayRegularizer] = ...,
        lr_ratio: Optional[Callable[[Tensor], float]] = ...,
        apply_decay_param_fun: Optional[Callable[[str], bool]] = ...,
        grad_clip: Optional[GradientClipBase] = ...,
        lazy_mode: bool = ...,
        multi_precision: bool = ...,
        name: Optional[str] = ...,
    ) -> None: ...



from __future__ import annotations

from typing import Any, Optional

from typing_extensions import Literal

from ..._typing import DataLayout1D, DataLayout2D, DataLayout3D, DataLayoutND, Tensor
from ..._typing.basic import IntSequence
from ...fluid.dygraph import BatchNorm as BatchNorm
from ...fluid.dygraph import SpectralNorm as SpectralNorm
from ...framework import ParamAttr
from .. import Layer

class _InstanceNormBase(Layer):
    scale: Any = ...
    bias: Any = ...
    def __init__(
        self,
        num_features: int,
        epsilon: float = ...,
        momentum: float = ...,
        weight_attr: Optional[ParamAttr] = ...,
        bias_attr: Optional[ParamAttr | bool] = ...,
        data_format: DataLayoutND = ...,
        name: Optional[str] = ...,
    ) -> None: ...
    def forward(self, input: Tensor) -> Tensor: ...
    __call__ = forward

class InstanceNorm1D(_InstanceNormBase):
    def __init__(
        self,
        num_features: int,
        epsilon: float = ...,
        momentum: float = ...,
        weight_attr: Optional[ParamAttr] = ...,
        bias_attr: Optional[ParamAttr | bool] = ...,
        data_format: DataLayout1D = ...,
        name: Optional[str] = ...,
    ) -> None: ...

class InstanceNorm2D(_InstanceNormBase):
    def __init__(
        self,
        num_features: int,
        epsilon: float = ...,
        momentum: float = ...,
        weight_attr: Optional[ParamAttr] = ...,
        bias_attr: Optional[ParamAttr | bool] = ...,
        data_format: DataLayout2D = ...,
        name: Optional[str] = ...,
    ) -> None: ...

class InstanceNorm3D(_InstanceNormBase):
    def __init__(
        self,
        num_features: int,
        epsilon: float = ...,
        momentum: float = ...,
        weight_attr: Optional[ParamAttr] = ...,
        bias_attr: Optional[ParamAttr | bool] = ...,
        data_format: DataLayout3D = ...,
        name: Optional[str] = ...,
    ) -> None: ...

class GroupNorm(Layer):
    weight: Any = ...
    bias: Any = ...
    def __init__(
        self,
        num_groups: int,
        num_channels: int,
        epsilon: float = ...,
        weight_attr: Optional[ParamAttr | bool] = ...,
        bias_attr: Optional[ParamAttr | bool] = ...,
        data_format: Literal["NCHW"] = ...,
        name: Optional[str] = ...,
    ) -> None: ...
    def forward(self, input: Tensor) -> Tensor: ...
    __call__ = forward

class LayerNorm(Layer):
    weight: Any = ...
    bias: Any = ...
    def __init__(
        self,
        normalized_shape: int | IntSequence,
        epsilon: float = ...,
        weight_attr: Optional[ParamAttr | bool] = ...,
        bias_attr: Optional[ParamAttr | bool] = ...,
        name: Optional[str] = ...,
    ) -> None: ...
    def forward(self, input: Tensor) -> Tensor: ...
    __call__ = forward

class _BatchNormBase(Layer):
    weight: Any = ...
    bias: Any = ...
    def __init__(
        self,
        num_features: int,
        momentum: float = ...,
        epsilon: float = ...,
        weight_attr: Optional[ParamAttr | bool] = ...,
        bias_attr: Optional[ParamAttr | bool] = ...,
        data_format: DataLayoutND = ...,
        use_global_stats: Optional[bool] = ...,
        name: Optional[str] = ...,
    ) -> None: ...
    def forward(self, input: Tensor) -> Tensor: ...
    __call__ = forward

class BatchNorm1D(_BatchNormBase):
    def __init__(
        self,
        num_features: int,
        momentum: float = ...,
        epsilon: float = ...,
        weight_attr: Optional[ParamAttr | bool] = ...,
        bias_attr: Optional[ParamAttr | bool] = ...,
        data_format: DataLayout1D = ...,
        use_global_stats: Optional[bool] = ...,
        name: Optional[str] = ...,
    ) -> None: ...

class BatchNorm2D(_BatchNormBase):
    def __init__(
        self,
        num_features: int,
        momentum: float = ...,
        epsilon: float = ...,
        weight_attr: Optional[ParamAttr | bool] = ...,
        bias_attr: Optional[ParamAttr | bool] = ...,
        data_format: DataLayout2D = ...,
        use_global_stats: Optional[bool] = ...,
        name: Optional[str] = ...,
    ) -> None: ...

class BatchNorm3D(_BatchNormBase):
    def __init__(
        self,
        num_features: int,
        momentum: float = ...,
        epsilon: float = ...,
        weight_attr: Optional[ParamAttr | bool] = ...,
        bias_attr: Optional[ParamAttr | bool] = ...,
        data_format: DataLayout3D = ...,
        use_global_stats: Optional[bool] = ...,
        name: Optional[str] = ...,
    ) -> None: ...

class SyncBatchNorm(_BatchNormBase):
    def __init__(
        self,
        num_features: int,
        momentum: float = ...,
        epsilon: float = ...,
        weight_attr: Optional[ParamAttr | bool] = ...,
        bias_attr: Optional[ParamAttr | bool] = ...,
        data_format: DataLayoutND = ...,
        name: Optional[str] = ...,
    ) -> None: ...
    @classmethod
    def convert_sync_batchnorm(cls, layer: _BatchNormBase) -> SyncBatchNorm: ...

class LocalResponseNorm(Layer):
    size: Any = ...
    alpha: Any = ...
    beta: Any = ...
    k: Any = ...
    data_format: Any = ...
    name: Any = ...
    def __init__(
        self,
        size: int,
        alpha: float = ...,
        beta: float = ...,
        k: float = ...,
        data_format: DataLayoutND = ...,
        name: Optional[str] = ...,
    ) -> None: ...
    def forward(self, input: Tensor) -> Tensor: ...
    __call__ = forward



from __future__ import annotations

from typing import Any, Optional

from .. import Tensor
from .._typing import DTypeLike, ShapeLike

def bernoulli(x: Any, name: Optional[str] = ...): ...
def poisson(x: Any, name: Optional[str] = ...): ...
def multinomial(x: Any, num_samples: int = ..., replacement: bool = ..., name: Optional[str] = ...): ...
def gaussian(
    shape: Any, mean: float = ..., std: float = ..., dtype: Optional[Any] = ..., name: Optional[str] = ...
): ...
def standard_normal(shape: Any, dtype: Optional[Any] = ..., name: Optional[str] = ...): ...
def randn(
    shape: Any,
    dtype: Optional[DTypeLike] = ...,
    name: Optional[str] = ...,
) -> Tensor: ...
def normal(mean: float = ..., std: float = ..., shape: Optional[Any] = ..., name: Optional[str] = ...): ...
def uniform(
    shape: ShapeLike,
    dtype: Optional[DTypeLike] = ...,
    min: float = ...,
    max: float = ...,
    seed: int = ...,
    name: Optional[str] = ...,
) -> Tensor: ...
def uniform_(
    x: Tensor,
    min: float = ...,
    max: float = ...,
    seed: int = ...,
    name: Optional[str] = ...,
) -> Tensor: ...
def randint(
    low: int = ...,
    high: Optional[int] = ...,
    shape: ShapeLike = ...,
    dtype: Optional[DTypeLike] = ...,
    name: Optional[str] = ...,
) -> Tensor: ...
def randint_like(
    x: Tensor,
    low: int = ...,
    high: Optional[int] = ...,
    dtype: Optional[DTypeLike] = ...,
    name: Optional[str] = ...,
) -> Tensor: ...
def randperm(n: Any, dtype: str = ..., name: Optional[str] = ...): ...
def rand(
    shape: ShapeLike,
    dtype: Optional[DTypeLike] = ...,
    name: Optional[str] = ...,
) -> Tensor: ...
def exponential_(x: Any, lam: float = ..., name: Optional[str] = ...): ...



from __future__ import annotations

from typing import Any, Optional

from .._typing import IntSequence, Tensor

def mean(
    x: Tensor,
    axis: Optional[int | IntSequence] = ...,
    keepdim: bool = ...,
    name: Optional[str] = ...,
) -> Tensor: ...
def var(x: Any, axis: Optional[Any] = ..., unbiased: bool = ..., keepdim: bool = ..., name: Optional[str] = ...): ...
def std(x: Any, axis: Optional[Any] = ..., unbiased: bool = ..., keepdim: bool = ..., name: Optional[str] = ...): ...
def numel(x: Any, name: Optional[str] = ...): ...
def median(x: Any, axis: Optional[Any] = ..., keepdim: bool = ..., name: Optional[str] = ...): ...
def quantile(x: Any, q: Any, axis: Optional[Any] = ..., keepdim: bool = ...): ...



from __future__ import annotations

from typing import Any, Optional

from typing_extensions import NotRequired, TypedDict

from .._typing import Tensor
from ..callbacks import Callback
from ..fluid.clip import GradientClipBase
from ..fluid.framework import Program
from ..fluid.regularizer import WeightDecayRegularizer
from .lr import LRScheduler as LRScheduler

OptimizerStateDict = dict[str, Tensor]

class ParameterConfig(TypedDict):
    params: list[Tensor]
    weight_decay: NotRequired[Optional[float | WeightDecayRegularizer]]
    learning_rate: NotRequired[Optional[float | Tensor | LRScheduler]]

class Optimizer:
    regularization: Any = ...
    helper: Any = ...
    clear_gradients: Any = ...
    def __init__(
        self,
        learning_rate: float | LRScheduler,
        parameters: Optional[list[Tensor] | tuple[Tensor, ...]] = ...,
        weight_decay: Optional[float | WeightDecayRegularizer] = ...,
        grad_clip: Optional[GradientClipBase] = ...,
        name: Optional[str] = ...,
    ) -> None: ...
    def state_dict(self) -> OptimizerStateDict: ...
    def set_state_dict(self, state_dict: OptimizerStateDict) -> None: ...
    def get_opti_var_name_list(self) -> list[str]: ...
    def set_lr(self, value: float) -> None: ...
    def get_lr(self) -> float: ...
    def backward(
        self,
        loss: Tensor,
        startup_program: Optional[Program] = ...,
        parameters: Optional[list[Tensor] | list[str]] = ...,
        no_grad_set: Optional[set[Tensor] | set[str]] = ...,
        callbacks: Optional[list[Callback]] = ...,
    ) -> list[tuple[Tensor, Tensor]]: ...
    def apply_gradients(self, params_grads: list[tuple[Tensor, Tensor]]) -> list[Any]: ...  # TODO: list[op]
    def append_regularization_ops(
        self,
        parameters_and_grads: list[tuple[Tensor, Tensor]],
        regularization: Optional[Any] = ...,  # TODO: WeightDecayRegularizer?
    ) -> list[tuple[Tensor, Tensor]]: ...
    def clear_grad(self, set_to_zero: bool = ...) -> None: ...
    def minimize(
        self,
        loss: Tensor,
        startup_program: Optional[Program] = ...,
        parameters: Optional[list[Tensor] | list[str]] = ...,
        no_grad_set: Optional[set[Tensor] | set[str]] = ...,
    ) -> list[tuple[Tensor, Tensor]]: ...
    def step(self) -> None: ...



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['paddlepaddle-stubs',
 'paddlepaddle-stubs.examples',
 'paddlepaddle-stubs.tests',
 'paddlepaddle-stubs.tests.test_nn',
 'paddlepaddle-stubs.tests.test_nn.test_layer',
 'paddlepaddle-stubs.tests.test_optimizer',
 'paddlepaddle-stubs.tests.test_tensor',
 'paddlepaddle-stubs.tests.test_vision',
 'paddlepaddle-stubs.tests.test_vision.test_models',
 'paddlepaddle-stubs.tests.test_vision.test_transforms']

package_data = \
{'': ['*'],
 'paddlepaddle-stubs': ['.git/*',
                        '.git/hooks/*',
                        '.git/info/*',
                        '.git/logs/*',
                        '.git/logs/refs/heads/*',
                        '.git/logs/refs/remotes/origin/*',
                        '.git/objects/00/*',
                        '.git/objects/01/*',
                        '.git/objects/02/*',
                        '.git/objects/03/*',
                        '.git/objects/04/*',
                        '.git/objects/05/*',
                        '.git/objects/06/*',
                        '.git/objects/07/*',
                        '.git/objects/08/*',
                        '.git/objects/09/*',
                        '.git/objects/0a/*',
                        '.git/objects/0b/*',
                        '.git/objects/0c/*',
                        '.git/objects/0d/*',
                        '.git/objects/0e/*',
                        '.git/objects/0f/*',
                        '.git/objects/10/*',
                        '.git/objects/11/*',
                        '.git/objects/12/*',
                        '.git/objects/13/*',
                        '.git/objects/14/*',
                        '.git/objects/15/*',
                        '.git/objects/16/*',
                        '.git/objects/17/*',
                        '.git/objects/18/*',
                        '.git/objects/19/*',
                        '.git/objects/1a/*',
                        '.git/objects/1b/*',
                        '.git/objects/1c/*',
                        '.git/objects/1d/*',
                        '.git/objects/1e/*',
                        '.git/objects/1f/*',
                        '.git/objects/20/*',
                        '.git/objects/21/*',
                        '.git/objects/22/*',
                        '.git/objects/23/*',
                        '.git/objects/24/*',
                        '.git/objects/25/*',
                        '.git/objects/26/*',
                        '.git/objects/27/*',
                        '.git/objects/28/*',
                        '.git/objects/29/*',
                        '.git/objects/2a/*',
                        '.git/objects/2b/*',
                        '.git/objects/2c/*',
                        '.git/objects/2d/*',
                        '.git/objects/2e/*',
                        '.git/objects/2f/*',
                        '.git/objects/30/*',
                        '.git/objects/31/*',
                        '.git/objects/32/*',
                        '.git/objects/33/*',
                        '.git/objects/34/*',
                        '.git/objects/35/*',
                        '.git/objects/36/*',
                        '.git/objects/37/*',
                        '.git/objects/38/*',
                        '.git/objects/39/*',
                        '.git/objects/3a/*',
                        '.git/objects/3b/*',
                        '.git/objects/3c/*',
                        '.git/objects/3d/*',
                        '.git/objects/3e/*',
                        '.git/objects/3f/*',
                        '.git/objects/40/*',
                        '.git/objects/41/*',
                        '.git/objects/42/*',
                        '.git/objects/43/*',
                        '.git/objects/44/*',
                        '.git/objects/45/*',
                        '.git/objects/46/*',
                        '.git/objects/47/*',
                        '.git/objects/48/*',
                        '.git/objects/49/*',
                        '.git/objects/4a/*',
                        '.git/objects/4b/*',
                        '.git/objects/4c/*',
                        '.git/objects/4d/*',
                        '.git/objects/4e/*',
                        '.git/objects/4f/*',
                        '.git/objects/50/*',
                        '.git/objects/51/*',
                        '.git/objects/52/*',
                        '.git/objects/53/*',
                        '.git/objects/54/*',
                        '.git/objects/55/*',
                        '.git/objects/56/*',
                        '.git/objects/57/*',
                        '.git/objects/58/*',
                        '.git/objects/59/*',
                        '.git/objects/5a/*',
                        '.git/objects/5b/*',
                        '.git/objects/5c/*',
                        '.git/objects/5d/*',
                        '.git/objects/5e/*',
                        '.git/objects/5f/*',
                        '.git/objects/60/*',
                        '.git/objects/61/*',
                        '.git/objects/62/*',
                        '.git/objects/63/*',
                        '.git/objects/64/*',
                        '.git/objects/65/*',
                        '.git/objects/66/*',
                        '.git/objects/67/*',
                        '.git/objects/68/*',
                        '.git/objects/69/*',
                        '.git/objects/6a/*',
                        '.git/objects/6b/*',
                        '.git/objects/6c/*',
                        '.git/objects/6d/*',
                        '.git/objects/6e/*',
                        '.git/objects/6f/*',
                        '.git/objects/70/*',
                        '.git/objects/71/*',
                        '.git/objects/72/*',
                        '.git/objects/73/*',
                        '.git/objects/74/*',
                        '.git/objects/75/*',
                        '.git/objects/76/*',
                        '.git/objects/77/*',
                        '.git/objects/78/*',
                        '.git/objects/79/*',
                        '.git/objects/7a/*',
                        '.git/objects/7b/*',
                        '.git/objects/7c/*',
                        '.git/objects/7d/*',
                        '.git/objects/7e/*',
                        '.git/objects/7f/*',
                        '.git/objects/80/*',
                        '.git/objects/81/*',
                        '.git/objects/82/*',
                        '.git/objects/83/*',
                        '.git/objects/84/*',
                        '.git/objects/85/*',
                        '.git/objects/86/*',
                        '.git/objects/87/*',
                        '.git/objects/88/*',
                        '.git/objects/89/*',
                        '.git/objects/8a/*',
                        '.git/objects/8b/*',
                        '.git/objects/8c/*',
                        '.git/objects/8d/*',
                        '.git/objects/8e/*',
                        '.git/objects/8f/*',
                        '.git/objects/90/*',
                        '.git/objects/91/*',
                        '.git/objects/92/*',
                        '.git/objects/93/*',
                        '.git/objects/94/*',
                        '.git/objects/95/*',
                        '.git/objects/96/*',
                        '.git/objects/97/*',
                        '.git/objects/98/*',
                        '.git/objects/99/*',
                        '.git/objects/9a/*',
                        '.git/objects/9b/*',
                        '.git/objects/9c/*',
                        '.git/objects/9d/*',
                        '.git/objects/9e/*',
                        '.git/objects/9f/*',
                        '.git/objects/a0/*',
                        '.git/objects/a1/*',
                        '.git/objects/a2/*',
                        '.git/objects/a3/*',
                        '.git/objects/a4/*',
                        '.git/objects/a5/*',
                        '.git/objects/a6/*',
                        '.git/objects/a7/*',
                        '.git/objects/a8/*',
                        '.git/objects/a9/*',
                        '.git/objects/aa/*',
                        '.git/objects/ab/*',
                        '.git/objects/ac/*',
                        '.git/objects/ad/*',
                        '.git/objects/ae/*',
                        '.git/objects/af/*',
                        '.git/objects/b0/*',
                        '.git/objects/b1/*',
                        '.git/objects/b2/*',
                        '.git/objects/b3/*',
                        '.git/objects/b4/*',
                        '.git/objects/b5/*',
                        '.git/objects/b6/*',
                        '.git/objects/b7/*',
                        '.git/objects/b8/*',
                        '.git/objects/b9/*',
                        '.git/objects/ba/*',
                        '.git/objects/bb/*',
                        '.git/objects/bc/*',
                        '.git/objects/bd/*',
                        '.git/objects/be/*',
                        '.git/objects/bf/*',
                        '.git/objects/c0/*',
                        '.git/objects/c1/*',
                        '.git/objects/c2/*',
                        '.git/objects/c3/*',
                        '.git/objects/c4/*',
                        '.git/objects/c5/*',
                        '.git/objects/c6/*',
                        '.git/objects/c7/*',
                        '.git/objects/c8/*',
                        '.git/objects/c9/*',
                        '.git/objects/ca/*',
                        '.git/objects/cb/*',
                        '.git/objects/cc/*',
                        '.git/objects/cd/*',
                        '.git/objects/ce/*',
                        '.git/objects/cf/*',
                        '.git/objects/d0/*',
                        '.git/objects/d1/*',
                        '.git/objects/d2/*',
                        '.git/objects/d3/*',
                        '.git/objects/d4/*',
                        '.git/objects/d5/*',
                        '.git/objects/d6/*',
                        '.git/objects/d7/*',
                        '.git/objects/d8/*',
                        '.git/objects/d9/*',
                        '.git/objects/da/*',
                        '.git/objects/db/*',
                        '.git/objects/dc/*',
                        '.git/objects/dd/*',
                        '.git/objects/de/*',
                        '.git/objects/df/*',
                        '.git/objects/e0/*',
                        '.git/objects/e1/*',
                        '.git/objects/e2/*',
                        '.git/objects/e3/*',
                        '.git/objects/e4/*',
                        '.git/objects/e5/*',
                        '.git/objects/e6/*',
                        '.git/objects/e7/*',
                        '.git/objects/e8/*',
                        '.git/objects/e9/*',
                        '.git/objects/ea/*',
                        '.git/objects/eb/*',
                        '.git/objects/ec/*',
                        '.git/objects/ed/*',
                        '.git/objects/ee/*',
                        '.git/objects/ef/*',
                        '.git/objects/f0/*',
                        '.git/objects/f1/*',
                        '.git/objects/f2/*',
                        '.git/objects/f3/*',
                        '.git/objects/f4/*',
                        '.git/objects/f5/*',
                        '.git/objects/f6/*',
                        '.git/objects/f7/*',
                        '.git/objects/f8/*',
                        '.git/objects/f9/*',
                        '.git/objects/fa/*',
                        '.git/objects/fb/*',
                        '.git/objects/fc/*',
                        '.git/objects/fd/*',
                        '.git/objects/fe/*',
                        '.git/objects/ff/*',
                        '.git/objects/pack/*',
                        '.git/refs/heads/*',
                        '.git/refs/remotes/origin/*',
                        '.git/refs/tags/*',
                        '.github/*',
                        '.github/workflows/*',
                        '.vscode/*',
                        'paddle-stubs/_C_ops.pyi',
                        'paddle-stubs/_C_ops.pyi',
                        'paddle-stubs/_C_ops.pyi',
                        'paddle-stubs/_C_ops.pyi',
                        'paddle-stubs/_C_ops.pyi',
                        'paddle-stubs/_C_ops.pyi',
                        'paddle-stubs/_C_ops.pyi',
                        'paddle-stubs/_C_ops.pyi',
                        'paddle-stubs/_C_ops.pyi',
                        'paddle-stubs/_C_ops.pyi',
                        'paddle-stubs/_C_ops.pyi',
                        'paddle-stubs/_C_ops.pyi',
                        'paddle-stubs/_C_ops.pyi',
                        'paddle-stubs/_C_ops.pyi',
                        'paddle-stubs/__init__.pyi',
                        'paddle-stubs/__init__.pyi',
                        'paddle-stubs/__init__.pyi',
                        'paddle-stubs/__init__.pyi',
                        'paddle-stubs/__init__.pyi',
                        'paddle-stubs/__init__.pyi',
                        'paddle-stubs/__init__.pyi',
                        'paddle-stubs/__init__.pyi',
                        'paddle-stubs/__init__.pyi',
                        'paddle-stubs/__init__.pyi',
                        'paddle-stubs/__init__.pyi',
                        'paddle-stubs/__init__.pyi',
                        'paddle-stubs/__init__.pyi',
                        'paddle-stubs/__init__.pyi',
                        'paddle-stubs/_typing/*',
                        'paddle-stubs/amp/*',
                        'paddle-stubs/autograd/*',
                        'paddle-stubs/batch.pyi',
                        'paddle-stubs/batch.pyi',
                        'paddle-stubs/batch.pyi',
                        'paddle-stubs/batch.pyi',
                        'paddle-stubs/batch.pyi',
                        'paddle-stubs/batch.pyi',
                        'paddle-stubs/batch.pyi',
                        'paddle-stubs/batch.pyi',
                        'paddle-stubs/batch.pyi',
                        'paddle-stubs/batch.pyi',
                        'paddle-stubs/batch.pyi',
                        'paddle-stubs/batch.pyi',
                        'paddle-stubs/batch.pyi',
                        'paddle-stubs/batch.pyi',
                        'paddle-stubs/callbacks.pyi',
                        'paddle-stubs/callbacks.pyi',
                        'paddle-stubs/callbacks.pyi',
                        'paddle-stubs/callbacks.pyi',
                        'paddle-stubs/callbacks.pyi',
                        'paddle-stubs/callbacks.pyi',
                        'paddle-stubs/callbacks.pyi',
                        'paddle-stubs/callbacks.pyi',
                        'paddle-stubs/callbacks.pyi',
                        'paddle-stubs/callbacks.pyi',
                        'paddle-stubs/callbacks.pyi',
                        'paddle-stubs/callbacks.pyi',
                        'paddle-stubs/callbacks.pyi',
                        'paddle-stubs/callbacks.pyi',
                        'paddle-stubs/check_import_scipy.pyi',
                        'paddle-stubs/check_import_scipy.pyi',
                        'paddle-stubs/check_import_scipy.pyi',
                        'paddle-stubs/check_import_scipy.pyi',
                        'paddle-stubs/check_import_scipy.pyi',
                        'paddle-stubs/check_import_scipy.pyi',
                        'paddle-stubs/check_import_scipy.pyi',
                        'paddle-stubs/check_import_scipy.pyi',
                        'paddle-stubs/check_import_scipy.pyi',
                        'paddle-stubs/check_import_scipy.pyi',
                        'paddle-stubs/check_import_scipy.pyi',
                        'paddle-stubs/check_import_scipy.pyi',
                        'paddle-stubs/check_import_scipy.pyi',
                        'paddle-stubs/check_import_scipy.pyi',
                        'paddle-stubs/common_ops_import.pyi',
                        'paddle-stubs/common_ops_import.pyi',
                        'paddle-stubs/common_ops_import.pyi',
                        'paddle-stubs/common_ops_import.pyi',
                        'paddle-stubs/common_ops_import.pyi',
                        'paddle-stubs/common_ops_import.pyi',
                        'paddle-stubs/common_ops_import.pyi',
                        'paddle-stubs/common_ops_import.pyi',
                        'paddle-stubs/common_ops_import.pyi',
                        'paddle-stubs/common_ops_import.pyi',
                        'paddle-stubs/common_ops_import.pyi',
                        'paddle-stubs/common_ops_import.pyi',
                        'paddle-stubs/common_ops_import.pyi',
                        'paddle-stubs/common_ops_import.pyi',
                        'paddle-stubs/compat.pyi',
                        'paddle-stubs/compat.pyi',
                        'paddle-stubs/compat.pyi',
                        'paddle-stubs/compat.pyi',
                        'paddle-stubs/compat.pyi',
                        'paddle-stubs/compat.pyi',
                        'paddle-stubs/compat.pyi',
                        'paddle-stubs/compat.pyi',
                        'paddle-stubs/compat.pyi',
                        'paddle-stubs/compat.pyi',
                        'paddle-stubs/compat.pyi',
                        'paddle-stubs/compat.pyi',
                        'paddle-stubs/compat.pyi',
                        'paddle-stubs/compat.pyi',
                        'paddle-stubs/cost_model/*',
                        'paddle-stubs/dataset/*',
                        'paddle-stubs/device/*',
                        'paddle-stubs/device/cuda/*',
                        'paddle-stubs/distributed/*',
                        'paddle-stubs/distributed/auto_parallel/*',
                        'paddle-stubs/distributed/auto_parallel/cost/*',
                        'paddle-stubs/distributed/auto_parallel/operators/*',
                        'paddle-stubs/distributed/auto_parallel/tuner/*',
                        'paddle-stubs/distributed/fleet/*',
                        'paddle-stubs/distributed/fleet/base/*',
                        'paddle-stubs/distributed/fleet/data_generator/*',
                        'paddle-stubs/distributed/fleet/dataset/*',
                        'paddle-stubs/distributed/fleet/elastic/*',
                        'paddle-stubs/distributed/fleet/meta_optimizers/*',
                        'paddle-stubs/distributed/fleet/meta_optimizers/ascend/*',
                        'paddle-stubs/distributed/fleet/meta_optimizers/dygraph_optimizer/*',
                        'paddle-stubs/distributed/fleet/meta_optimizers/sharding/*',
                        'paddle-stubs/distributed/fleet/meta_parallel/*',
                        'paddle-stubs/distributed/fleet/meta_parallel/parallel_layers/*',
                        'paddle-stubs/distributed/fleet/meta_parallel/pp_utils/*',
                        'paddle-stubs/distributed/fleet/meta_parallel/sharding/*',
                        'paddle-stubs/distributed/fleet/metrics/*',
                        'paddle-stubs/distributed/fleet/runtime/*',
                        'paddle-stubs/distributed/fleet/utils/*',
                        'paddle-stubs/distributed/launch/*',
                        'paddle-stubs/distributed/launch/context/*',
                        'paddle-stubs/distributed/launch/controllers/*',
                        'paddle-stubs/distributed/launch/job/*',
                        'paddle-stubs/distributed/launch/plugins/*',
                        'paddle-stubs/distributed/launch/utils/*',
                        'paddle-stubs/distributed/metric/*',
                        'paddle-stubs/distributed/models/*',
                        'paddle-stubs/distributed/models/moe/*',
                        'paddle-stubs/distributed/passes/*',
                        'paddle-stubs/distributed/ps/*',
                        'paddle-stubs/distributed/ps/utils/*',
                        'paddle-stubs/distributed/sharding/*',
                        'paddle-stubs/distribution/*',
                        'paddle-stubs/fft.pyi',
                        'paddle-stubs/fft.pyi',
                        'paddle-stubs/fft.pyi',
                        'paddle-stubs/fft.pyi',
                        'paddle-stubs/fft.pyi',
                        'paddle-stubs/fft.pyi',
                        'paddle-stubs/fft.pyi',
                        'paddle-stubs/fft.pyi',
                        'paddle-stubs/fft.pyi',
                        'paddle-stubs/fft.pyi',
                        'paddle-stubs/fft.pyi',
                        'paddle-stubs/fft.pyi',
                        'paddle-stubs/fft.pyi',
                        'paddle-stubs/fft.pyi',
                        'paddle-stubs/fluid/*',
                        'paddle-stubs/fluid/contrib/*',
                        'paddle-stubs/fluid/contrib/decoder/*',
                        'paddle-stubs/fluid/contrib/extend_optimizer/*',
                        'paddle-stubs/fluid/contrib/layers/*',
                        'paddle-stubs/fluid/contrib/mixed_precision/*',
                        'paddle-stubs/fluid/contrib/mixed_precision/bf16/*',
                        'paddle-stubs/fluid/contrib/quantize/*',
                        'paddle-stubs/fluid/contrib/slim/*',
                        'paddle-stubs/fluid/contrib/slim/quantization/*',
                        'paddle-stubs/fluid/contrib/slim/quantization/imperative/*',
                        'paddle-stubs/fluid/contrib/sparsity/*',
                        'paddle-stubs/fluid/dataloader/*',
                        'paddle-stubs/fluid/distributed/*',
                        'paddle-stubs/fluid/dygraph/*',
                        'paddle-stubs/fluid/dygraph/amp/*',
                        'paddle-stubs/fluid/dygraph/dygraph_to_static/*',
                        'paddle-stubs/fluid/incubate/*',
                        'paddle-stubs/fluid/incubate/checkpoint/*',
                        'paddle-stubs/fluid/incubate/data_generator/*',
                        'paddle-stubs/fluid/incubate/fleet/*',
                        'paddle-stubs/fluid/incubate/fleet/base/*',
                        'paddle-stubs/fluid/incubate/fleet/collective/*',
                        'paddle-stubs/fluid/incubate/fleet/parameter_server/*',
                        'paddle-stubs/fluid/incubate/fleet/parameter_server/distribute_transpiler/*',
                        'paddle-stubs/fluid/incubate/fleet/parameter_server/ir/*',
                        'paddle-stubs/fluid/incubate/fleet/parameter_server/pslib/*',
                        'paddle-stubs/fluid/incubate/fleet/utils/*',
                        'paddle-stubs/fluid/inference/*',
                        'paddle-stubs/fluid/layers/*',
                        'paddle-stubs/fluid/transpiler/*',
                        'paddle-stubs/fluid/transpiler/details/*',
                        'paddle-stubs/framework/*',
                        'paddle-stubs/hapi/*',
                        'paddle-stubs/hub.pyi',
                        'paddle-stubs/hub.pyi',
                        'paddle-stubs/hub.pyi',
                        'paddle-stubs/hub.pyi',
                        'paddle-stubs/hub.pyi',
                        'paddle-stubs/hub.pyi',
                        'paddle-stubs/hub.pyi',
                        'paddle-stubs/hub.pyi',
                        'paddle-stubs/hub.pyi',
                        'paddle-stubs/hub.pyi',
                        'paddle-stubs/hub.pyi',
                        'paddle-stubs/hub.pyi',
                        'paddle-stubs/hub.pyi',
                        'paddle-stubs/hub.pyi',
                        'paddle-stubs/incubate/*',
                        'paddle-stubs/incubate/autograd/*',
                        'paddle-stubs/incubate/checkpoint/*',
                        'paddle-stubs/incubate/multiprocessing/*',
                        'paddle-stubs/incubate/nn/*',
                        'paddle-stubs/incubate/nn/functional/*',
                        'paddle-stubs/incubate/nn/layer/*',
                        'paddle-stubs/incubate/operators/*',
                        'paddle-stubs/incubate/optimizer/*',
                        'paddle-stubs/incubate/optimizer/functional/*',
                        'paddle-stubs/incubate/passes/*',
                        'paddle-stubs/incubate/tensor/*',
                        'paddle-stubs/io/*',
                        'paddle-stubs/jit/*',
                        'paddle-stubs/jit/dy2static/*',
                        'paddle-stubs/libs/*',
                        'paddle-stubs/linalg.pyi',
                        'paddle-stubs/linalg.pyi',
                        'paddle-stubs/linalg.pyi',
                        'paddle-stubs/linalg.pyi',
                        'paddle-stubs/linalg.pyi',
                        'paddle-stubs/linalg.pyi',
                        'paddle-stubs/linalg.pyi',
                        'paddle-stubs/linalg.pyi',
                        'paddle-stubs/linalg.pyi',
                        'paddle-stubs/linalg.pyi',
                        'paddle-stubs/linalg.pyi',
                        'paddle-stubs/linalg.pyi',
                        'paddle-stubs/linalg.pyi',
                        'paddle-stubs/linalg.pyi',
                        'paddle-stubs/metric/*',
                        'paddle-stubs/nn/*',
                        'paddle-stubs/nn/functional/*',
                        'paddle-stubs/nn/initializer/*',
                        'paddle-stubs/nn/layer/*',
                        'paddle-stubs/nn/quant/*',
                        'paddle-stubs/nn/utils/*',
                        'paddle-stubs/onnx/*',
                        'paddle-stubs/optimizer/*',
                        'paddle-stubs/profiler/*',
                        'paddle-stubs/reader/*',
                        'paddle-stubs/reader/tests/*',
                        'paddle-stubs/regularizer.pyi',
                        'paddle-stubs/regularizer.pyi',
                        'paddle-stubs/regularizer.pyi',
                        'paddle-stubs/regularizer.pyi',
                        'paddle-stubs/regularizer.pyi',
                        'paddle-stubs/regularizer.pyi',
                        'paddle-stubs/regularizer.pyi',
                        'paddle-stubs/regularizer.pyi',
                        'paddle-stubs/regularizer.pyi',
                        'paddle-stubs/regularizer.pyi',
                        'paddle-stubs/regularizer.pyi',
                        'paddle-stubs/regularizer.pyi',
                        'paddle-stubs/regularizer.pyi',
                        'paddle-stubs/regularizer.pyi',
                        'paddle-stubs/signal.pyi',
                        'paddle-stubs/signal.pyi',
                        'paddle-stubs/signal.pyi',
                        'paddle-stubs/signal.pyi',
                        'paddle-stubs/signal.pyi',
                        'paddle-stubs/signal.pyi',
                        'paddle-stubs/signal.pyi',
                        'paddle-stubs/signal.pyi',
                        'paddle-stubs/signal.pyi',
                        'paddle-stubs/signal.pyi',
                        'paddle-stubs/signal.pyi',
                        'paddle-stubs/signal.pyi',
                        'paddle-stubs/signal.pyi',
                        'paddle-stubs/signal.pyi',
                        'paddle-stubs/sparse/*',
                        'paddle-stubs/sparse/functional/*',
                        'paddle-stubs/sparse/layer/*',
                        'paddle-stubs/static/*',
                        'paddle-stubs/static/amp/*',
                        'paddle-stubs/static/nn/*',
                        'paddle-stubs/static/sparsity/*',
                        'paddle-stubs/sysconfig.pyi',
                        'paddle-stubs/sysconfig.pyi',
                        'paddle-stubs/sysconfig.pyi',
                        'paddle-stubs/sysconfig.pyi',
                        'paddle-stubs/sysconfig.pyi',
                        'paddle-stubs/sysconfig.pyi',
                        'paddle-stubs/sysconfig.pyi',
                        'paddle-stubs/sysconfig.pyi',
                        'paddle-stubs/sysconfig.pyi',
                        'paddle-stubs/sysconfig.pyi',
                        'paddle-stubs/sysconfig.pyi',
                        'paddle-stubs/sysconfig.pyi',
                        'paddle-stubs/sysconfig.pyi',
                        'paddle-stubs/sysconfig.pyi',
                        'paddle-stubs/tensor/*',
                        'paddle-stubs/text/*',
                        'paddle-stubs/text/datasets/*',
                        'paddle-stubs/utils/*',
                        'paddle-stubs/utils/cpp_extension/*',
                        'paddle-stubs/utils/gast/*',
                        'paddle-stubs/version.pyi',
                        'paddle-stubs/version.pyi',
                        'paddle-stubs/version.pyi',
                        'paddle-stubs/version.pyi',
                        'paddle-stubs/version.pyi',
                        'paddle-stubs/version.pyi',
                        'paddle-stubs/version.pyi',
                        'paddle-stubs/version.pyi',
                        'paddle-stubs/version.pyi',
                        'paddle-stubs/version.pyi',
                        'paddle-stubs/version.pyi',
                        'paddle-stubs/version.pyi',
                        'paddle-stubs/version.pyi',
                        'paddle-stubs/version.pyi',
                        'paddle-stubs/vision/*',
                        'paddle-stubs/vision/datasets/*',
                        'paddle-stubs/vision/models/*',
                        'paddle-stubs/vision/transforms/*']}

install_requires = \
['paddlepaddle>=2.3.0,<3.0.0', 'typing-extensions>=4.3.0,<5.0.0']

setup_kwargs = {
    'name': 'paddlepaddle-stubs',
    'version': '2.3.1a4',
    'description': '',
    'long_description': '# PaddlePaddle Stubs <sup>WIP</sup>\n\nA stubs package as described in [PEP 561](https://peps.python.org/pep-0561/) for [PaddlePaddle](https://github.com/PaddlePaddle/Paddle).\n\n<p align="center">\n   <a href="https://python.org/" target="_blank"><img alt="PyPI - Python Version" src="https://img.shields.io/pypi/pyversions/paddlepaddle-stubs?logo=python&style=flat-square"></a>\n   <a href="https://pypi.org/project/paddlepaddle-stubs/" target="_blank"><img src="https://img.shields.io/pypi/v/paddlepaddle-stubs?style=flat-square" alt="pypi"></a>\n   <a href="https://pypi.org/project/paddlepaddle-stubs/" target="_blank"><img alt="PyPI - Downloads" src="https://img.shields.io/pypi/dm/paddlepaddle-stubs?style=flat-square"></a>\n   <a href="LICENSE"><img alt="LICENSE" src="https://img.shields.io/github/license/cattidea/paddlepaddle-stubs?style=flat-square"></a>\n   <a href="https://github.com/psf/black"><img alt="black" src="https://img.shields.io/badge/code%20style-black-000000?style=flat-square"></a>\n   <a href="https://gitmoji.dev"><img src="https://img.shields.io/badge/gitmoji-%20😜%20😍-FFDD67?style=flat-square" alt="Gitmoji"></a>\n</p>\n\n目前仅针对 [Pyright](https://github.com/microsoft/pyright) 进行了编写和测试，尚未支持 [Mypy](https://github.com/python/mypy)（Mypy 相对于 Pyright 太难用了，缺失功能太多），因此目前可能仅仅与 VS Code 的 Pylance 扩展一起工作良好～\n\n> **Note**\n>\n> 本 repo 非运行时库，因此本 repo 中所有示例和单测可能在运行时并不能正常运行，仅仅是为了更加全面和方便对类型进行检查而已。\n\n## Usage\n\n```bash\npip install paddlepaddle-stubs --pre\n```\n\n此时再打开编辑器，查看编辑器的类型提示～～～\n\nVS Code 推荐配置：\n\n```jsonc\n{\n   "python.languageServer": "Pylance",\n   // 现在有一些类型在 strict mode 工作的并不是很好，推荐先使用 basic mode\n   "python.analysis.typeCheckingMode": "basic",\n   "python.analysis.inlayHints.functionReturnTypes": true,\n   "python.analysis.inlayHints.variableTypes": true\n}\n```\n\n## Status\n\n起步阶段，最低支持 Python3.7，目前基于 PaddlePaddle 2.3.1 开发，第一个可用版本应当是 2.3.1 版本或者更晚的 2.4 版本～\n\n## Roadmap\n\nSee [paddlepaddle-stubs 2.3.1 Roadmap](https://github.com/orgs/cattidea/projects/3)\n\n### 2.3.1 alpha\n\n> **Note**\n>\n> 此时本 repo 可 public，并发布到 PyPI\n\n-  [x] 通过 basic mode 的类型检查\n-  [x] 完善全部 P1 级别的类型信息（strict 下工作良好）\n\n### 2.3.1 beta\n\n-  [ ] 完善全部 P2 级别的类型信息（strict 下工作良好）\n-  [ ] [官网示例高层 API 搭建 MNIST 示例代码](./examples/mnist_example.py)在 strict 模式下工作良好\n\n### Future\n\n之后应该是一边升级到新版本（2.4.0），一边继续完善 P3、P4、P5 级别的类型信息（strict 下工作良好）\n\n要添加的还有很多，各个 API 类型信息需要仔细填写（修改自动生成的 `Any`，添加返回值类型），有兴趣的小伙伴可以一起来参与呀～\n',
    'author': 'Nyakku Shigure',
    'author_email': 'sigure.qaq@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/cattidea/paddlepaddle-stubs',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from __future__ import annotations

from typing import Any, Optional

from ..._typing import DataLayout1D, DataLayout2D, DataLayout3D, IntSequence, Tensor
from .. import Layer
from .conv import PaddingSizeStr

class AvgPool1D(Layer):
    kernel_size: Any = ...
    stride: Any = ...
    padding: Any = ...
    ceil_mode: Any = ...
    exclusive: Any = ...
    name: Any = ...
    def __init__(
        self,
        kernel_size: int | IntSequence,
        stride: Optional[int | IntSequence] = ...,
        padding: int | IntSequence | PaddingSizeStr = ...,
        exclusive: bool = ...,
        ceil_mode: bool = ...,
        name: Optional[str] = ...,
    ) -> None: ...
    def forward(self, x: Tensor) -> Tensor: ...
    __call__ = forward

class AvgPool2D(Layer):
    ksize: Any = ...
    stride: Any = ...
    padding: Any = ...
    ceil_mode: Any = ...
    exclusive: Any = ...
    divisor: Any = ...
    data_format: Any = ...
    name: Any = ...
    def __init__(
        self,
        kernel_size: int | IntSequence,
        stride: Optional[int | IntSequence] = ...,
        padding: int | IntSequence | PaddingSizeStr = ...,
        ceil_mode: bool = ...,
        exclusive: bool = ...,
        divisor_override: Optional[float] = ...,
        data_format: DataLayout2D = ...,
        name: Optional[str] = ...,
    ) -> None: ...
    def forward(self, x: Tensor) -> Tensor: ...
    __call__ = forward

class AvgPool3D(Layer):
    ksize: Any = ...
    stride: Any = ...
    padding: Any = ...
    ceil_mode: Any = ...
    exclusive: Any = ...
    divisor: Any = ...
    data_format: Any = ...
    name: Any = ...
    def __init__(
        self,
        kernel_size: int | IntSequence,
        stride: Optional[int | IntSequence] = ...,
        padding: int | IntSequence | PaddingSizeStr = ...,
        ceil_mode: bool = ...,
        exclusive: bool = ...,
        divisor_override: Optional[float] = ...,
        data_format: DataLayout3D = ...,
        name: Optional[str] = ...,
    ) -> None: ...
    def forward(self, x: Tensor) -> Tensor: ...
    __call__ = forward

class MaxPool1D(Layer):
    kernel_size: Any = ...
    stride: Any = ...
    padding: Any = ...
    ceil_mode: Any = ...
    return_mask: Any = ...
    name: Any = ...
    def __init__(
        self,
        kernel_size: int | IntSequence,
        stride: Optional[int | IntSequence] = ...,
        padding: int | IntSequence | PaddingSizeStr = ...,
        return_mask: bool = ...,
        ceil_mode: bool = ...,
        name: Optional[str] = ...,
    ) -> None: ...
    def forward(self, x: Tensor) -> Tensor: ...
    __call__ = forward

class MaxPool2D(Layer):
    ksize: Any = ...
    stride: Any = ...
    padding: Any = ...
    return_mask: Any = ...
    ceil_mode: Any = ...
    data_format: Any = ...
    name: Any = ...
    def __init__(
        self,
        kernel_size: int | IntSequence,
        stride: Optional[int | IntSequence] = ...,
        padding: int | IntSequence | PaddingSizeStr = ...,
        return_mask: bool = ...,
        ceil_mode: bool = ...,
        data_format: DataLayout2D = ...,
        name: Optional[str] = ...,
    ) -> None: ...
    def forward(self, x: Tensor) -> Tensor: ...
    __call__ = forward

class MaxPool3D(Layer):
    ksize: Any = ...
    stride: Any = ...
    padding: Any = ...
    return_mask: Any = ...
    ceil_mode: Any = ...
    data_format: Any = ...
    name: Any = ...
    def __init__(
        self,
        kernel_size: int | IntSequence,
        stride: Optional[int | IntSequence] = ...,
        padding: int | IntSequence | PaddingSizeStr = ...,
        return_mask: bool = ...,
        ceil_mode: bool = ...,
        data_format: DataLayout3D = ...,
        name: Optional[str] = ...,
    ) -> None: ...
    def forward(self, x: Tensor) -> Tensor: ...
    __call__ = forward

class AdaptiveAvgPool1D(Layer):
    output_size: Any = ...
    name: Any = ...
    def __init__(self, output_size: int | IntSequence, name: Optional[str] = ...) -> None: ...
    def forward(self, input: Tensor) -> Tensor: ...
    __call__ = forward

class AdaptiveAvgPool2D(Layer):
    def __init__(
        self,
        output_size: int | IntSequence,
        data_format: DataLayout2D = ...,
        name: Optional[str] = ...,
    ) -> None: ...
    def forward(self, x: Tensor) -> Tensor: ...
    __call__ = forward

class AdaptiveAvgPool3D(Layer):
    def __init__(
        self,
        output_size: int | IntSequence,
        data_format: DataLayout3D = ...,
        name: Optional[str] = ...,
    ) -> None: ...
    def forward(self, x: Tensor) -> Tensor: ...
    __call__ = forward

class AdaptiveMaxPool1D(Layer):
    output_size: Any = ...
    return_mask: Any = ...
    name: Any = ...
    def __init__(
        self,
        output_size: int | IntSequence,
        return_mask: bool = ...,
        name: Optional[str] = ...,
    ) -> None: ...
    def forward(self, input: Tensor) -> Tensor: ...
    __call__ = forward

class AdaptiveMaxPool2D(Layer):
    def __init__(
        self,
        output_size: int | IntSequence,
        return_mask: bool = ...,
        name: Optional[str] = ...,
    ) -> None: ...
    def forward(self, x: Tensor) -> Tensor: ...
    __call__ = forward

class AdaptiveMaxPool3D(Layer):
    def __init__(
        self,
        output_size: int | IntSequence,
        return_mask: bool = ...,
        name: Optional[str] = ...,
    ) -> None: ...
    def forward(self, x: Tensor) -> Tensor: ...
    __call__ = forward

class MaxUnPool1D(Layer):
    ksize: Any = ...
    stride: Any = ...
    padding: Any = ...
    data_format: Any = ...
    output_size: Any = ...
    name: Any = ...
    def __init__(
        self,
        kernel_size: int | IntSequence,
        stride: Optional[int | IntSequence] = ...,
        padding: int | IntSequence = ...,
        data_format: DataLayout1D = ...,
        output_size: Optional[IntSequence] = ...,
        name: Optional[str] = ...,
    ) -> None: ...
    def forward(self, x: Tensor, indices: Tensor) -> Tensor: ...
    __call__ = forward

class MaxUnPool2D(Layer):
    ksize: Any = ...
    stride: Any = ...
    padding: Any = ...
    data_format: Any = ...
    output_size: Any = ...
    name: Any = ...
    def __init__(
        self,
        kernel_size: int | IntSequence,
        stride: Optional[int | IntSequence] = ...,
        padding: int | IntSequence = ...,
        data_format: DataLayout2D = ...,
        output_size: Optional[IntSequence] = ...,
        name: Optional[str] = ...,
    ) -> None: ...
    def forward(self, x: Tensor, indices: Tensor) -> Tensor: ...
    __call__ = forward

class MaxUnPool3D(Layer):
    ksize: Any = ...
    stride: Any = ...
    padding: Any = ...
    data_format: Any = ...
    output_size: Any = ...
    name: Any = ...
    def __init__(
        self,
        kernel_size: int | IntSequence,
        stride: Optional[int | IntSequence] = ...,
        padding: int | IntSequence = ...,
        data_format: DataLayout3D = ...,
        output_size: Optional[IntSequence] = ...,
        name: Optional[str] = ...,
    ) -> None: ...
    def forward(self, x: Tensor, indices: Tensor) -> Tensor: ...
    __call__ = forward



from __future__ import annotations

from typing import Any, Optional

from paddle.fluid import core as core

def graph_sample_neighbors(
    row: Any,
    colptr: Any,
    input_nodes: Any,
    eids: Optional[Any] = ...,
    perm_buffer: Optional[Any] = ...,
    sample_size: int = ...,
    return_eids: bool = ...,
    flag_perm_buffer: bool = ...,
    name: Optional[str] = ...,
): ...



from __future__ import annotations

from collections.abc import Hashable, Iterable, Iterator, Sized
from typing import Any, Optional, SupportsFloat, SupportsInt

import numpy as np
import numpy.typing as npt
from typing_extensions import SupportsIndex

from .basic import Numberic
from .device import Place
from .dtype import DTypeLike, dtype
from .shape import ShapeLike

# `builtins.PyCapsule` unfortunately lacks annotations as of the moment;
# use `Any` as a stopgap measure
# @see also: https://github.com/numpy/numpy/blob/b6a3e837785eac58a2f68e126f4db7895ca047b3/numpy/__init__.pyi#L1465
_PyCapsule = Any

TensorLike = list["TensorLike"] | tuple["TensorLike", ...] | npt.NDArray[Any] | Tensor | Numberic

class Tensor(Sized, Iterable[Tensor], Hashable, SupportsFloat, SupportsInt, SupportsIndex):
    shape: list[int]
    dtype: dtype
    grad: npt.NDArray[Any]
    is_leaf: bool
    name: str
    ndim: int
    persistable: bool
    place: Place
    stop_gradient: bool
    T: Tensor
    __array_ufunc__: None

    # math operations
    def __neg__(self) -> Tensor: ...
    # def __pos__(self) -> Tensor: ... # missing
    def __add__(self, other: TensorLike) -> Tensor: ...
    def __radd__(self, other: TensorLike) -> Tensor: ...
    def __sub__(self, other: TensorLike) -> Tensor: ...
    def __rsub__(self, other: TensorLike) -> Tensor: ...
    def __mul__(self, other: TensorLike) -> Tensor: ...
    def __rmul__(self, other: TensorLike) -> Tensor: ...
    def __div__(self, other: TensorLike) -> Tensor: ...
    def __rdiv__(self, other: TensorLike) -> Tensor: ...
    def __truediv__(self, other: TensorLike) -> Tensor: ...
    def __rtruediv__(self, other: TensorLike) -> Tensor: ...
    def __floordiv__(self, other: TensorLike) -> Tensor: ...
    def __rfloordiv__(self, other: TensorLike) -> Tensor: ...
    def __mod__(self, other: TensorLike) -> Tensor: ...
    def __rmod__(self, other: TensorLike) -> Tensor: ...
    def __pow__(self, other: TensorLike) -> Tensor: ...
    def __rpow__(self, other: TensorLike) -> Tensor: ...
    def __matmul__(self, other: TensorLike) -> Tensor: ...
    def __rmatmul__(self, other: TensorLike) -> Tensor: ...

    # comparison operations
    def __eq__(self, other: TensorLike) -> Tensor: ...
    def __ne__(self, other: TensorLike) -> Tensor: ...
    def __le__(self, other: TensorLike) -> Tensor: ...
    def __ge__(self, other: TensorLike) -> Tensor: ...
    def __lt__(self, other: TensorLike) -> Tensor: ...
    def __gt__(self, other: TensorLike) -> Tensor: ...

    # bitwise operations
    def __or__(self, other: TensorLike) -> Tensor: ...
    def __ror__(self, other: TensorLike) -> Tensor: ...
    def __and__(self, other: TensorLike) -> Tensor: ...
    def __rand__(self, other: TensorLike) -> Tensor: ...
    def __xor__(self, other: TensorLike) -> Tensor: ...
    def __rxor__(self, other: TensorLike) -> Tensor: ...
    def __invert__(self) -> Tensor: ...
    # def __lshift__(self, other: TensorLike) -> Tensor: ... # missing
    # def __rshift__(self, other: TensorLike) -> Tensor: ... # missing
    # def __rlshift__(self, other: TensorLike) -> Tensor: ... # missing
    # def __rrshift__(self, other: TensorLike) -> Tensor: ... # missing

    # ?: Missing all inplace operations

    # others
    # def __abs__(self) -> Tensor: ... # missing
    def __repr__(self) -> str: ...
    def __str__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[Tensor]: ...
    def __getitem__(self, key: int | Tensor | slice) -> Tensor: ...
    def __float__(self) -> float: ...
    def __int__(self) -> int: ...
    def __nonzero__(self) -> bool: ...
    def __index__(self) -> int: ...
    def __hash__(self) -> int: ...
    def __contains__(self, other: TensorLike) -> bool: ...
    # def __dlpack__(self) -> _PyCapsule: ... # missing
    # def __dlpack_device__(self, stream: Optional[Any] = ...) # missing

    # methods
    def clear_grad(self) -> None: ...
    clear_gradient = clear_grad
    def item(self, *args: int) -> Numberic: ...
    def astype(self, dtype: DTypeLike) -> Tensor: ...
    def reshape(self, shape: ShapeLike, name: Optional[str] = ...) -> Tensor: ...
    def set_value(self, value: TensorLike) -> None: ...
    def numpy(self) -> npt.NDArray[Any]: ...
    def backward(
        self,
        grad_tensor: Optional[Tensor] = ...,
        retain_graph: bool = ...,
    ) -> None: ...
    def clone(self) -> Tensor: ...
    def broadcast_to(self, shape: ShapeLike, name: Optional[str] = ...) -> Tensor: ...
    def cast(self, dtype: DTypeLike) -> Tensor: ...
    def size(self) -> Tensor: ...
    def dim(self) -> int: ...
    def ndimension(self) -> int: ...
    def imag(self) -> Tensor: ...
    def real(self) -> Tensor: ...

    # extended math ops
    # TODO: ...



from __future__ import annotations

class Place: ...
class CPUPlace(Place): ...

class CUDAPlace(Place):
    def __init__(self, id: int) -> None: ...

class CUDAPinnedPlace(Place): ...

class NPUPlace(Place):
    def __init__(self, id: int) -> None: ...

class IPUPlace(Place): ...

class CustomPlace(Place):
    def __init__(self, name: str, id: int) -> None: ...

class MLUPlace(Place):
    def __init__(self, id: int) -> None: ...

class XPUPlace(Place):
    def __init__(self, id: int) -> None: ...

PlaceLike = (
    CPUPlace | CUDAPlace | CUDAPinnedPlace | NPUPlace | IPUPlace | CustomPlace | MLUPlace | XPUPlace | str
)  # TODO: only support the literal like "dev:id"
